@@ -3,7 +3,6 @@ import os
 import weakref
 from dataclasses import dataclass
 from itertools import product, takewhile
-from typing import List, Tuple
 from unittest import mock
 
 import numpy as np
@@ -25,7 +24,6 @@ from napari._tests.utils import (
     skip_on_win_ci,
 )
 from napari._vispy._tests.utils import vispy_image_scene_size
-from napari._vispy.utils.gl import fix_data_dtype
 from napari.components.viewer_model import ViewerModel
 from napari.layers import Labels, Points
 from napari.settings import get_settings
@@ -35,6 +33,16 @@ from napari.utils.theme import available_themes
 
 BUILTINS_DISP = 'napari'
 BUILTINS_NAME = 'builtins'
+NUMPY_INTEGER_TYPES = [
+    np.int8,
+    np.int16,
+    np.int32,
+    np.int64,
+    np.uint8,
+    np.uint16,
+    np.uint32,
+    np.uint64,
+]
 
 
 def test_qt_viewer(make_napari_viewer):
@@ -73,24 +81,24 @@ def test_qt_viewer_toggle_console(make_napari_viewer):
 
 
 @skip_local_popups
-@pytest.mark.skipif(os.environ.get("MIN_REQ", "0") == "1", reason="min req")
+@pytest.mark.skipif(os.environ.get('MIN_REQ', '0') == '1', reason='min req')
 def test_qt_viewer_console_focus(qtbot, make_napari_viewer):
     """Test console has focus when instantiating from viewer."""
     viewer = make_napari_viewer(show=True)
     view = viewer.window._qt_viewer
-    assert not view.console.hasFocus(), "console has focus before being shown"
+    assert not view.console.hasFocus(), 'console has focus before being shown'
 
     view.toggle_console_visibility(None)
 
     def console_has_focus():
         assert (
             view.console.hasFocus()
-        ), "console does not have focus when shown"
+        ), 'console does not have focus when shown'
 
     qtbot.waitUntil(console_has_focus)
 
 
-@pytest.mark.parametrize('layer_class, data, ndim', layer_test_data)
+@pytest.mark.parametrize(('layer_class', 'data', 'ndim'), layer_test_data)
 def test_add_layer(make_napari_viewer, layer_class, data, ndim):
     viewer = make_napari_viewer(ndisplay=int(np.clip(ndim, 2, 3)))
     view = viewer.window._qt_viewer
@@ -200,7 +208,7 @@ def test_z_order_adding_removing_images(make_napari_viewer):
     data = np.ones((10, 10))
 
     viewer = make_napari_viewer()
-    vis = viewer.window._qt_viewer.layer_to_visual
+    vis = viewer.window._qt_viewer.canvas.layer_to_visual
     viewer.add_image(data, colormap='red', name='red')
     viewer.add_image(data, colormap='green', name='green')
     viewer.add_image(data, colormap='blue', name='blue')
@@ -256,12 +264,53 @@ def test_screenshot(make_napari_viewer):
 
     # Take screenshot
     with pytest.warns(FutureWarning):
-        screenshot = viewer.window.qt_viewer.screenshot(flash=False)
+        viewer.window.qt_viewer.screenshot(flash=False)
     screenshot = viewer.window.screenshot(flash=False, canvas_only=True)
     assert screenshot.ndim == 3
 
 
-@pytest.mark.skip("new approach")
+def test_export_figure(make_napari_viewer, tmp_path):
+    viewer = make_napari_viewer()
+
+    np.random.seed(0)
+    # Add image
+    data = np.random.randint(150, 250, size=(250, 250))
+    layer = viewer.add_image(data)
+
+    camera_center = viewer.camera.center
+    camera_zoom = viewer.camera.zoom
+    img = viewer.export_figure(flash=False, path=str(tmp_path / 'img.png'))
+
+    assert viewer.camera.center == camera_center
+    assert viewer.camera.zoom == camera_zoom
+    assert img.shape == (250, 250, 4)
+    assert np.all(img != np.array([0, 0, 0, 0]))
+
+    assert (tmp_path / 'img.png').exists()
+
+    layer.scale = [0.12, 0.24]
+    img = viewer.export_figure(flash=False)
+    # allclose accounts for rounding errors when computing size in hidpi aka
+    # retina displays
+    np.testing.assert_allclose(img.shape, (250, 499, 4), atol=1)
+
+    layer.scale = [0.12, 0.12]
+    img = viewer.export_figure(flash=False)
+    assert img.shape == (250, 250, 4)
+
+    viewer.camera.center = [100, 100]
+    camera_center = viewer.camera.center
+    camera_zoom = viewer.camera.zoom
+    img = viewer.export_figure()
+
+    assert viewer.camera.center == camera_center
+    assert viewer.camera.zoom == camera_zoom
+    assert img.shape == (250, 250, 4)
+    assert np.all(img != np.array([0, 0, 0, 0]))
+    viewer.close()
+
+
+@pytest.mark.skip('new approach')
 def test_screenshot_dialog(make_napari_viewer, tmpdir):
     """Test save screenshot functionality."""
     viewer = make_napari_viewer()
@@ -290,9 +339,10 @@ def test_screenshot_dialog(make_napari_viewer, tmpdir):
     # Save screenshot
     input_filepath = os.path.join(tmpdir, 'test-save-screenshot')
     mock_return = (input_filepath, '')
-    with mock.patch('napari._qt._qt_viewer.QFileDialog') as mocker, mock.patch(
-        'napari._qt._qt_viewer.QMessageBox'
-    ) as mocker2:
+    with (
+        mock.patch('napari._qt._qt_viewer.QFileDialog') as mocker,
+        mock.patch('napari._qt._qt_viewer.QMessageBox') as mocker2,
+    ):
         mocker.getSaveFileName.return_value = mock_return
         mocker2.warning.return_value = QMessageBox.Yes
         viewer.window._qt_viewer._screenshot_dialog()
@@ -304,45 +354,6 @@ def test_screenshot_dialog(make_napari_viewer, tmpdir):
     assert np.allclose(output_data, expected_data)
 
 
-@pytest.mark.parametrize(
-    "dtype",
-    [
-        'int8',
-        'uint8',
-        'int16',
-        'uint16',
-        'int32',
-        'float16',
-        'float32',
-        'float64',
-    ],
-)
-def test_qt_viewer_data_integrity(make_napari_viewer, dtype):
-    """Test that the viewer doesn't change the underlying array."""
-    image = np.random.rand(10, 32, 32)
-    image *= 200 if dtype.endswith('8') else 2**14
-    image = image.astype(dtype)
-    imean = image.mean()
-
-    viewer = make_napari_viewer()
-    layer = viewer.add_image(image.copy())
-    data = layer.data
-
-    datamean = np.mean(data)
-    assert datamean == imean
-    # toggle dimensions
-    viewer.dims.ndisplay = 3
-    datamean = np.mean(data)
-    assert datamean == imean
-    # back to 2D
-    viewer.dims.ndisplay = 2
-    datamean = np.mean(data)
-    assert datamean == imean
-    # also check that vispy gets (almost) the same data
-    datamean = np.mean(fix_data_dtype(data))
-    assert np.allclose(datamean, imean, rtol=5e-04)
-
-
 def test_points_layer_display_correct_slice_on_scale(make_napari_viewer):
     viewer = make_napari_viewer()
     data = np.zeros((60, 60, 60))
@@ -350,9 +361,10 @@ def test_points_layer_display_correct_slice_on_scale(make_napari_viewer):
     pts = viewer.add_points(name='test', size=1, ndim=3)
     pts.add((8.7, 0, 0))
     viewer.dims.set_point(0, 30 * 0.29)  # middle plane
-    layer = viewer.layers[1]
-    indices, scale = layer._slice_data(layer._slice_indices)
-    np.testing.assert_equal(indices, [0])
+
+    request = pts._make_slice_request(viewer.dims)
+    response = request()
+    np.testing.assert_equal(response.indices, [0])
 
 
 @skip_on_win_ci
@@ -377,12 +389,12 @@ def test_qt_viewer_clipboard_with_flash(make_napari_viewer, qtbot):
         viewer.window._qt_viewer._welcome_widget.graphicsEffect() is not None
     )
     assert hasattr(
-        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, '_flash_animation'
     )
     qtbot.wait(500)  # wait for the animation to finish
     assert viewer.window._qt_viewer._welcome_widget.graphicsEffect() is None
     assert not hasattr(
-        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, '_flash_animation'
     )
 
     # clear clipboard and grab image from application view
@@ -397,10 +409,10 @@ def test_qt_viewer_clipboard_with_flash(make_napari_viewer, qtbot):
 
     # ensure the flash effect is applied
     assert viewer.window._qt_window.graphicsEffect() is not None
-    assert hasattr(viewer.window._qt_window, "_flash_animation")
+    assert hasattr(viewer.window._qt_window, '_flash_animation')
     qtbot.wait(500)  # wait for the animation to finish
     assert viewer.window._qt_window.graphicsEffect() is None
-    assert not hasattr(viewer.window._qt_window, "_flash_animation")
+    assert not hasattr(viewer.window._qt_window, '_flash_animation')
 
 
 @skip_on_win_ci
@@ -423,7 +435,7 @@ def test_qt_viewer_clipboard_without_flash(make_napari_viewer):
     # ensure the flash effect is not applied
     assert viewer.window._qt_viewer._welcome_widget.graphicsEffect() is None
     assert not hasattr(
-        viewer.window._qt_viewer._welcome_widget, "_flash_animation"
+        viewer.window._qt_viewer._welcome_widget, '_flash_animation'
     )
 
     # clear clipboard and grab image from application view
@@ -438,7 +450,7 @@ def test_qt_viewer_clipboard_without_flash(make_napari_viewer):
 
     # ensure the flash effect is not applied
     assert viewer.window._qt_window.graphicsEffect() is None
-    assert not hasattr(viewer.window._qt_window, "_flash_animation")
+    assert not hasattr(viewer.window._qt_window, '_flash_animation')
 
 
 def test_active_keybindings(make_napari_viewer):
@@ -473,7 +485,7 @@ def test_active_keybindings(make_napari_viewer):
 @dataclass
 class MouseEvent:
     # mock mouse event class
-    pos: List[int]
+    pos: list[int]
 
 
 def test_process_mouse_event(make_napari_viewer):
@@ -498,11 +510,11 @@ def test_process_mouse_event(make_napari_viewer):
         np.testing.assert_array_equal(event.dims_displayed, [1, 2, 3])
         assert event.dims_point[0] == data.shape[0] // 2
 
-        expected_position = view._map_canvas2world(new_pos)
+        expected_position = view.canvas._map_canvas2world(new_pos)
         np.testing.assert_almost_equal(expected_position, list(event.position))
 
     viewer.dims.ndisplay = 3
-    view._process_mouse_event(mouse_press_callbacks, mouse_event)
+    view.canvas._process_mouse_event(mouse_press_callbacks, mouse_event)
 
 
 @skip_local_popups
@@ -552,7 +564,7 @@ def test_leaks_labels(qtbot, make_napari_viewer):
     assert not dr()
 
 
-@pytest.mark.parametrize("theme", available_themes())
+@pytest.mark.parametrize('theme', available_themes())
 def test_canvas_color(make_napari_viewer, theme):
     """Test instantiating viewer with different themes.
 
@@ -594,7 +606,7 @@ def test_mixed_2d_and_3d_layers(make_napari_viewer, multiscale):
     img = np.ones((512, 256))
     # canvas size must be large enough that img fits in the canvas
     canvas_size = tuple(3 * s for s in img.shape)
-    expected_corner_pixels = np.asarray([[0, 0], [img.shape[0], img.shape[1]]])
+    expected_corner_pixels = np.asarray([[0, 0], [s - 1 for s in img.shape]])
 
     vol = np.stack([img] * 8, axis=0)
     if multiscale:
@@ -605,16 +617,22 @@ def test_mixed_2d_and_3d_layers(make_napari_viewer, multiscale):
 
     viewer.dims.order = (0, 1, 2)
     viewer.window._qt_viewer.canvas.size = canvas_size
-    viewer.window._qt_viewer.on_draw(None)
-    assert np.all(img_multi_layer.corner_pixels == expected_corner_pixels)
+    viewer.window._qt_viewer.canvas.on_draw(None)
+    np.testing.assert_array_equal(
+        img_multi_layer.corner_pixels, expected_corner_pixels
+    )
 
     viewer.dims.order = (2, 0, 1)
-    viewer.window._qt_viewer.on_draw(None)
-    assert np.all(img_multi_layer.corner_pixels == expected_corner_pixels)
+    viewer.window._qt_viewer.canvas.on_draw(None)
+    np.testing.assert_array_equal(
+        img_multi_layer.corner_pixels, expected_corner_pixels
+    )
 
     viewer.dims.order = (1, 2, 0)
-    viewer.window._qt_viewer.on_draw(None)
-    assert np.all(img_multi_layer.corner_pixels == expected_corner_pixels)
+    viewer.window._qt_viewer.canvas.on_draw(None)
+    np.testing.assert_array_equal(
+        img_multi_layer.corner_pixels, expected_corner_pixels
+    )
 
 
 def test_remove_add_image_3D(make_napari_viewer):
@@ -682,8 +700,8 @@ def test_insert_layer_ordering(make_napari_viewer):
     viewer.layers.append(pl1)
     viewer.layers.insert(0, pl2)
 
-    pl1_vispy = viewer.window._qt_viewer.layer_to_visual[pl1].node
-    pl2_vispy = viewer.window._qt_viewer.layer_to_visual[pl2].node
+    pl1_vispy = viewer.window._qt_viewer.canvas.layer_to_visual[pl1].node
+    pl2_vispy = viewer.window._qt_viewer.canvas.layer_to_visual[pl2].node
     assert pl1_vispy.order == 1
     assert pl2_vispy.order == 0
 
@@ -709,7 +727,7 @@ def _update_data(
     qtbot: QtBot,
     qt_viewer: QtViewer,
     dtype: np.dtype = np.uint64,
-) -> Tuple[np.ndarray, np.ndarray]:
+) -> tuple[np.ndarray, np.ndarray]:
     """Change layer data and return color of label and middle pixel of screenshot."""
     layer.data = np.full((2, 2), label, dtype=dtype)
     layer.selected_label = label
@@ -741,9 +759,9 @@ def qt_viewer_with_controls(qtbot):
 @skip_local_popups
 @skip_on_win_ci
 @pytest.mark.parametrize(
-    "use_selection", [True, False], ids=["selected", "all"]
+    'use_selection', [True, False], ids=['selected', 'all']
 )
-@pytest.mark.parametrize("dtype", [np.int8, np.int16, np.int64])
+@pytest.mark.parametrize('dtype', [np.int8, np.int16, np.int64])
 def test_label_colors_matching_widget_auto(
     qtbot, qt_viewer_with_controls, use_selection, dtype
 ):
@@ -776,7 +794,7 @@ def test_label_colors_matching_widget_auto(
         )
 
         npt.assert_allclose(
-            color_box_color, middle_pixel, atol=1, err_msg=f"label {label}"
+            color_box_color, middle_pixel, atol=1, err_msg=f'label {label}'
         )
         # there is a difference of rounding between the QtColorBox and the screenshot
 
@@ -784,9 +802,9 @@ def test_label_colors_matching_widget_auto(
 @skip_local_popups
 @skip_on_win_ci
 @pytest.mark.parametrize(
-    "use_selection", [True, False], ids=["selected", "all"]
+    'use_selection', [True, False], ids=['selected', 'all']
 )
-@pytest.mark.parametrize("dtype", [np.uint64, np.uint16, np.uint8, np.int16])
+@pytest.mark.parametrize('dtype', [np.uint64, np.uint16, np.uint8, np.int16])
 def test_label_colors_matching_widget_direct(
     qtbot, qt_viewer_with_controls, use_selection, dtype
 ):
@@ -795,16 +813,16 @@ def test_label_colors_matching_widget_direct(
 
     test_colors = (1, 2, 3, 8, 150, 50)
     color = {
-        0: "transparent",
-        1: "yellow",
-        3: "blue",
-        8: "red",
-        150: "green",
-        None: "white",
+        0: 'transparent',
+        1: 'yellow',
+        3: 'blue',
+        8: 'red',
+        150: 'green',
+        None: 'white',
     }
     if np.iinfo(dtype).min < 0:
-        color[-1] = "pink"
-        color[-2] = "orange"
+        color[-1] = 'pink'
+        color[-2] = 'orange'
         test_colors = test_colors + (-1, -2, -10)
 
     colormap = DirectLabelColormap(color_dict=color)
@@ -824,21 +842,21 @@ def test_label_colors_matching_widget_direct(
             layer, label, qtbot, qt_viewer_with_controls, dtype
         )
         npt.assert_almost_equal(
-            color_box_color, middle_pixel, err_msg=f"{label=}"
+            color_box_color, middle_pixel, err_msg=f'{label=}'
         )
         npt.assert_almost_equal(
             color_box_color,
             colormap.color_dict.get(label, colormap.color_dict[None]) * 255,
-            err_msg=f"{label=}",
+            err_msg=f'{label=}',
         )
 
 
-def test_axes_labels(make_napari_viewer):
+def test_axis_labels(make_napari_viewer):
     viewer = make_napari_viewer(ndisplay=3)
     layer = viewer.add_image(np.zeros((2, 2, 2)), scale=(1, 2, 4))
 
     layer_visual = viewer._window._qt_viewer.layer_to_visual[layer]
-    axes_visual = viewer._window._qt_viewer.overlay_to_visual[
+    axes_visual = viewer._window._qt_viewer.canvas._overlay_to_visual[
         viewer._overlays['axes']
     ]
 
@@ -859,7 +877,7 @@ def qt_viewer(qtbot):
     del qt_viewer
 
 
-def _find_margin(data: np.ndarray, additional_margin: int) -> Tuple[int, int]:
+def _find_margin(data: np.ndarray, additional_margin: int) -> tuple[int, int]:
     """
     helper function to determine margins in test_thumbnail_labels
     """
@@ -876,8 +894,7 @@ def _find_margin(data: np.ndarray, additional_margin: int) -> Tuple[int, int]:
 
 # @pytest.mark.xfail(reason="Fails on CI, but not locally")
 @skip_local_popups
-@skip_on_win_ci
-@pytest.mark.parametrize('direct', [True, False], ids=["direct", "auto"])
+@pytest.mark.parametrize('direct', [True, False], ids=['direct', 'auto'])
 def test_thumbnail_labels(qtbot, direct, qt_viewer: QtViewer, tmp_path):
     # Add labels to empty viewer
     layer = qt_viewer.viewer.add_labels(
@@ -904,30 +921,29 @@ def test_thumbnail_labels(qtbot, direct, qt_viewer: QtViewer, tmp_path):
 
     import imageio
 
-    imageio.imwrite(tmp_path / "canvas_screenshot_.png", canvas_screenshot_)
-    np.savez(tmp_path / "canvas_screenshot_.npz", canvas_screenshot_)
+    imageio.imwrite(tmp_path / 'canvas_screenshot_.png', canvas_screenshot_)
+    np.savez(tmp_path / 'canvas_screenshot_.npz', canvas_screenshot_)
 
     # cut off black border
     margin1, margin2 = _find_margin(canvas_screenshot_, 10)
     canvas_screenshot = canvas_screenshot_[margin1:-margin1, margin2:-margin2]
     assert (
         canvas_screenshot.size > 0
-    ), f"{canvas_screenshot_.shape}, {margin1=}, {margin2=}"
+    ), f'{canvas_screenshot_.shape}, {margin1=}, {margin2=}'
 
     thumbnail = layer.thumbnail
     scaled_thumbnail = ndi.zoom(
         thumbnail,
         np.array(canvas_screenshot.shape) / np.array(thumbnail.shape),
         order=0,
-        mode="nearest",
+        mode='nearest',
     )
     close = np.isclose(canvas_screenshot, scaled_thumbnail)
     problematic_pixels_count = np.sum(~close)
     assert problematic_pixels_count < 0.01 * canvas_screenshot.size
 
 
-@skip_on_win_ci
-@pytest.mark.parametrize("dtype", [np.int8, np.int16, np.int32])
+@pytest.mark.parametrize('dtype', [np.int8, np.int16, np.int32])
 def test_background_color(qtbot, qt_viewer: QtViewer, dtype):
     data = np.zeros((10, 10), dtype=dtype)
     data[5:] = 10
@@ -948,10 +964,10 @@ def test_background_color(qtbot, qt_viewer: QtViewer, dtype):
         npt.assert_array_equal(
             background_pixel,
             [0, 0, 0, 255],
-            err_msg=f"background {background}",
+            err_msg=f'background {background}',
         )
         npt.assert_array_equal(
-            color_pixel, color, err_msg=f"background {background}"
+            color_pixel, color, err_msg=f'background {background}'
         )
 
 
@@ -959,7 +975,7 @@ def test_rendering_interpolation(qtbot, qt_viewer):
     data = np.zeros((20, 20, 20), dtype=np.uint8)
     data[1:-1, 1:-1, 1:-1] = 5
     layer = qt_viewer.viewer.add_labels(
-        data, opacity=1, rendering="translucent"
+        data, opacity=1, rendering='translucent'
     )
     layer.selected_label = 5
     qt_viewer.viewer.dims.ndisplay = 3
@@ -976,7 +992,7 @@ def test_shortcut_passing(make_napari_viewer):
     layer = viewer.add_labels(
         np.zeros((2, 2, 2), dtype=np.uint8), scale=(1, 2, 4)
     )
-    layer.mode = "fill"
+    layer.mode = 'fill'
 
     qt_window = viewer.window._qt_window
 
@@ -990,23 +1006,22 @@ def test_shortcut_passing(make_napari_viewer):
             QEvent.Type.KeyPress, Qt.Key.Key_1, Qt.KeyboardModifier.NoModifier
         )
     )
-    assert layer.mode == "erase"
+    assert layer.mode == 'erase'
 
 
-@skip_on_win_ci
-@pytest.mark.parametrize("mode", ["direct", "random"])
+@pytest.mark.parametrize('mode', ['direct', 'random'])
 def test_selection_collision(qt_viewer: QtViewer, mode):
     data = np.zeros((10, 10), dtype=np.uint8)
     data[:5] = 10
     data[5:] = 10 + 49
     layer = qt_viewer.viewer.add_labels(data, opacity=1)
     layer.selected_label = 10
-    if mode == "direct":
+    if mode == 'direct':
         layer.colormap = DirectLabelColormap(
-            color_dict={10: "red", 10 + 49: "red", None: "black"}
+            color_dict={10: 'red', 10 + 49: 'red', None: 'black'}
         )
 
-    for dtype in np.sctypes['int'] + np.sctypes['uint']:
+    for dtype in NUMPY_INTEGER_TYPES:
         layer.data = data.astype(dtype)
         layer.show_selected_label = False
         QApplication.processEvents()
@@ -1014,7 +1029,7 @@ def test_selection_collision(qt_viewer: QtViewer, mode):
         shape = np.array(canvas_screenshot.shape[:2])
         pixel_10 = canvas_screenshot[tuple((shape * 0.25).astype(int))]
         pixel_59 = canvas_screenshot[tuple((shape * 0.75).astype(int))]
-        npt.assert_array_equal(pixel_10, pixel_59, err_msg=f"{dtype}")
+        npt.assert_array_equal(pixel_10, pixel_59, err_msg=f'{dtype}')
         assert not np.all(pixel_10 == [0, 0, 0, 255]), dtype
 
         layer.show_selected_label = True
@@ -1024,27 +1039,27 @@ def test_selection_collision(qt_viewer: QtViewer, mode):
         pixel_10_2 = canvas_screenshot[tuple((shape * 0.25).astype(int))]
         pixel_59_2 = canvas_screenshot[tuple((shape * 0.75).astype(int))]
 
-        npt.assert_array_equal(pixel_59_2, [0, 0, 0, 255], err_msg=f"{dtype}")
-        npt.assert_array_equal(pixel_10_2, pixel_10, err_msg=f"{dtype}")
+        npt.assert_array_equal(pixel_59_2, [0, 0, 0, 255], err_msg=f'{dtype}')
+        npt.assert_array_equal(pixel_10_2, pixel_10, err_msg=f'{dtype}')
 
 
 def test_all_supported_dtypes(qt_viewer):
     data = np.zeros((10, 10), dtype=np.uint8)
-    layer = qt_viewer.viewer.add_labels(data, opacity=1)
+    layer_ = qt_viewer.viewer.add_labels(data, opacity=1)
 
-    for i, dtype in enumerate(np.sctypes['int'] + np.sctypes['uint'], start=1):
+    for i, dtype in enumerate(NUMPY_INTEGER_TYPES, start=1):
         data = np.full((10, 10), i, dtype=dtype)
-        layer.data = data
+        layer_.data = data
         QApplication.processEvents()
         canvas_screenshot = qt_viewer.screenshot(flash=False)
         midd_pixel = canvas_screenshot[
             tuple(np.array(canvas_screenshot.shape[:2]) // 2)
         ]
         npt.assert_equal(
-            midd_pixel, layer.colormap.map(i) * 255, err_msg=f"{dtype} {i}"
+            midd_pixel, layer_.colormap.map(i) * 255, err_msg=f'{dtype=} {i=}'
         )
 
-    layer.colormap = DirectLabelColormap(
+    layer_.colormap = DirectLabelColormap(
         color_dict={
             0: 'red',
             1: 'green',
@@ -1062,21 +1077,21 @@ def test_all_supported_dtypes(qt_viewer):
         }
     )
 
-    for i, dtype in enumerate(np.sctypes['int'] + np.sctypes['uint'], start=1):
+    for i, dtype in enumerate(NUMPY_INTEGER_TYPES, start=1):
         data = np.full((10, 10), i, dtype=dtype)
-        layer.data = data
+        layer_.data = data
         QApplication.processEvents()
         canvas_screenshot = qt_viewer.screenshot(flash=False)
         midd_pixel = canvas_screenshot[
             tuple(np.array(canvas_screenshot.shape[:2]) // 2)
         ]
         npt.assert_equal(
-            midd_pixel, layer.colormap.map(i) * 255, err_msg=f"{dtype} {i}"
+            midd_pixel, layer_.colormap.map(i) * 255, err_msg=f'{dtype} {i}'
         )
 
 
 def test_more_than_uint16_colors(qt_viewer):
-    pytest.importorskip("numba")
+    pytest.importorskip('numba')
     # this test is slow (10s locally)
     data = np.zeros((10, 10), dtype=np.uint32)
     colors = {
@@ -1100,5 +1115,136 @@ def test_more_than_uint16_colors(qt_viewer):
             tuple(np.array(canvas_screenshot.shape[:2]) // 2)
         ]
         npt.assert_equal(
-            midd_pixel, layer.colormap.map(i) * 255, err_msg=f"{i}"
+            midd_pixel, layer.colormap.map(i) * 255, err_msg=f'{i}'
         )
+
+
+def test_points_2d_to_3d(make_napari_viewer):
+    """See https://github.com/napari/napari/issues/6925"""
+    # this requires a full viewer cause some issues are caused only by
+    # qt processing events
+    viewer = make_napari_viewer(ndisplay=2, show=True)
+    viewer.add_points()
+    QApplication.processEvents()
+    viewer.dims.ndisplay = 3
+    QApplication.processEvents()
+
+
+@skip_local_popups
+def test_scale_bar_colored(qt_viewer, qtbot):
+    viewer = qt_viewer.viewer
+    scale_bar = viewer.scale_bar
+
+    # Add black image
+    data = np.zeros((2, 2))
+    viewer.add_image(data)
+
+    # Check scale bar is not visible (all the canvas is black - `[0, 0, 0, 255]`)
+    def check_all_black():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert np.all(screenshot == [0, 0, 0, 255], axis=-1).all()
+
+    qtbot.waitUntil(check_all_black)
+
+    # Check scale bar is visible (canvas has white `[1, 1, 1, 255]` in it)
+    def check_white_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert not np.all(screenshot == [0, 0, 0, 255], axis=-1).all()
+        assert np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+
+    scale_bar.visible = True
+    qtbot.waitUntil(check_white_scale_bar)
+
+    # Check scale bar is colored (canvas has fuchsia `[1, 0, 1, 255]` and not white in it)
+    def check_colored_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert not np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+        assert np.all(screenshot == [1, 0, 1, 255], axis=-1).any()
+
+    scale_bar.colored = True
+    qtbot.waitUntil(check_colored_scale_bar)
+
+    # Check scale bar is still visible but not colored (canvas has white again but not fuchsia in it)
+    def check_only_white_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+        assert not np.all(screenshot == [1, 0, 1, 255], axis=-1).any()
+
+    scale_bar.colored = False
+    qtbot.waitUntil(check_only_white_scale_bar)
+
+
+@skip_local_popups
+def test_scale_bar_ticks(qt_viewer, qtbot):
+    viewer = qt_viewer.viewer
+    scale_bar = viewer.scale_bar
+
+    # Add black image
+    data = np.zeros((2, 2))
+    viewer.add_image(data)
+
+    # Check scale bar is not visible (all the canvas is black - `[0, 0, 0, 255]`)
+    def check_all_black():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert np.all(screenshot == [0, 0, 0, 255], axis=-1).all()
+
+    qtbot.waitUntil(check_all_black)
+
+    # Check scale bar is visible (canvas has white `[1, 1, 1, 255]` in it)
+    def check_white_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert not np.all(screenshot == [0, 0, 0, 255], axis=-1).all()
+        assert np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+
+    scale_bar.visible = True
+    qtbot.waitUntil(check_white_scale_bar)
+
+    # Check scale bar has ticks active and take screenshot for later comparison
+    assert scale_bar.ticks
+    screenshot_with_ticks = qt_viewer.screenshot(flash=False)
+
+    # Check scale bar without ticks (still white present but new screenshot differs from ticks one)
+    def check_no_ticks_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+        npt.assert_raises(
+            AssertionError,
+            npt.assert_array_equal,
+            screenshot,
+            screenshot_with_ticks,
+        )
+
+    scale_bar.ticks = False
+    qtbot.waitUntil(check_no_ticks_scale_bar)
+
+    # Check scale bar again has ticks (still white present and new screenshot corresponds with ticks one)
+    def check_ticks_scale_bar():
+        screenshot = qt_viewer.screenshot(flash=False)
+        assert np.all(screenshot == [1, 1, 1, 255], axis=-1).any()
+        npt.assert_array_equal(screenshot, screenshot_with_ticks)
+
+    scale_bar.ticks = True
+    qtbot.waitUntil(check_ticks_scale_bar)
+
+
+@skip_local_popups
+def test_dask_cache(qt_viewer):
+    initial_dask_cache = get_settings().application.dask.cache
+
+    # check that disabling dask cache setting calls related logic
+    with mock.patch(
+        'napari._qt.qt_viewer.resize_dask_cache'
+    ) as mock_resize_dask_cache:
+        get_settings().application.dask.enabled = False
+    mock_resize_dask_cache.assert_called_once_with(
+        int(int(False) * initial_dask_cache * 1e9)
+    )
+
+    # check that enabling dask cache setting calls related logic
+    with mock.patch(
+        'napari._qt.qt_viewer.resize_dask_cache'
+    ) as mock_resize_dask_cache:
+        get_settings().application.dask.enabled = True
+    mock_resize_dask_cache.assert_called_once_with(
+        int(int(True) * initial_dask_cache * 1e9)
+    )
