@@ -1,19 +1,21 @@
 from __future__ import annotations
 
 from collections.abc import MutableSequence
-from typing import TYPE_CHECKING, Any, Generic, Tuple, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Generic, TypeVar, Union
 
 from qtpy.QtCore import QAbstractItemModel, QModelIndex, Qt
 
-from ...utils.events import disconnect_events
-from ...utils.events.containers import SelectableEventedList
-from ...utils.translations import trans
+from napari.utils.events import disconnect_events
+from napari.utils.events.containers import SelectableEventedList
+from napari.utils.translations import trans
 
 if TYPE_CHECKING:
-    from qtpy.QtWidgets import QWidget
+    from typing import Optional
+
+    from qtpy.QtWidgets import QWidget  # type: ignore[attr-defined]
 
 
-ItemType = TypeVar("ItemType")
+ItemType = TypeVar('ItemType')
 
 ItemRole = Qt.UserRole
 SortRole = Qt.UserRole + 1
@@ -75,8 +77,10 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
     # ########## Reimplemented Public Qt Functions ##################
 
     def __init__(
-        self, root: SelectableEventedList[ItemType], parent: QWidget = None
-    ):
+        self,
+        root: SelectableEventedList[ItemType],
+        parent: Optional[QWidget] = None,
+    ) -> None:
         super().__init__(parent=parent)
         self.setRoot(root)
 
@@ -132,7 +136,7 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
         """
         return 1
 
-    def rowCount(self, parent: QModelIndex = None) -> int:
+    def rowCount(self, parent: Optional[QModelIndex] = None) -> int:
         """Returns the number of rows under the given parent.
 
         When the parent is valid it means that rowCount is returning the number
@@ -146,7 +150,7 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
             return 0
 
     def index(
-        self, row: int, column: int = 0, parent: QModelIndex = None
+        self, row: int, column: int = 0, parent: Optional[QModelIndex] = None
     ) -> QModelIndex:
         """Return a QModelIndex for item at `row`, `column` and `parent`."""
 
@@ -197,12 +201,12 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
         if not isinstance(root, SelectableEventedList):
             raise TypeError(
                 trans._(
-                    "root must be an instance of {class_name}",
+                    'root must be an instance of {class_name}',
                     deferred=True,
                     class_name=SelectableEventedList,
                 )
             )
-        current_root = getattr(self, "_root", None)
+        current_root = getattr(self, '_root', None)
         if root is current_root:
             return
 
@@ -220,8 +224,8 @@ class _BaseEventedItemModel(QAbstractItemModel, Generic[ItemType]):
         self._root.events.connect(self._process_event)
 
     def _split_nested_index(
-        self, nested_index: Union[int, Tuple[int, ...]]
-    ) -> Tuple[QModelIndex, int]:
+        self, nested_index: Union[int, tuple[int, ...]]
+    ) -> tuple[QModelIndex, int]:
         """Return (parent_index, row) for a given index."""
         if isinstance(nested_index, int):
             return QModelIndex(), nested_index
