@@ -3,6 +3,7 @@ from unittest import mock
 
 import numpy as np
 import pytest
+import zarr
 from npe2 import DynamicPlugin
 from npe2.manifest.contributions import SampleDataURI
 from qtpy.QtWidgets import QLabel, QRadioButton
@@ -13,6 +14,8 @@ from napari._qt.dialogs.qt_reader_dialog import (
     open_with_dialog_choices,
     prepare_remaining_readers,
 )
+from napari._qt.qt_viewer import QtViewer
+from napari.components import ViewerModel
 from napari.errors.reader_errors import ReaderPluginError
 from napari.settings import get_settings
 
@@ -164,13 +167,14 @@ def test_open_sample_data_shows_all_readers(
     )
 
 
-def test_open_with_dialog_choices_persist(
-    builtins, make_napari_viewer, tmp_path
-):
+def test_open_with_dialog_choices_persist(builtins, tmp_path, qtbot):
     pth = tmp_path / 'my-file.npy'
     np.save(pth, np.random.random((10, 10)))
 
-    viewer = make_napari_viewer()
+    viewer = ViewerModel()
+    qt_viewer = QtViewer(viewer)
+    qtbot.addWidget(qt_viewer)
+
     open_with_dialog_choices(
         display_name=builtins.display_name,
         persist=True,
@@ -178,13 +182,41 @@ def test_open_with_dialog_choices_persist(
         readers={builtins.name: builtins.display_name},
         paths=[str(pth)],
         stack=False,
-        qt_viewer=viewer.window._qt_viewer,
+        qt_viewer=qt_viewer,
     )
     assert len(viewer.layers) == 1
     # make sure extension was saved with *
     assert get_settings().plugins.extension2reader['*.npy'] == builtins.name
 
 
+def test_open_with_dialog_choices_persist_dir(builtins, tmp_path, qtbot):
+    pth = tmp_path / 'data.zarr'
+    z = zarr.open(
+        str(pth), mode='w', shape=(10, 10), chunks=(5, 5), dtype='f4'
+    )
+    z[:] = np.random.random((10, 10))
+
+    viewer = ViewerModel()
+    qt_viewer = QtViewer(viewer)
+    qtbot.addWidget(qt_viewer)
+
+    open_with_dialog_choices(
+        display_name=builtins.display_name,
+        persist=True,
+        extension=str(pth),
+        readers={builtins.name: builtins.display_name},
+        paths=[str(pth)],
+        stack=False,
+        qt_viewer=qt_viewer,
+    )
+    assert len(viewer.layers) == 1
+    # make sure extension was saved without * and with trailing slash
+    assert (
+        get_settings().plugins.extension2reader[f'{pth}{os.sep}']
+        == builtins.name
+    )
+
+
 def test_open_with_dialog_choices_raises(make_napari_viewer):
     viewer = make_napari_viewer()
 
