@@ -3,17 +3,26 @@ import pytest
 
 from napari._qt.layer_controls.qt_labels_controls import QtLabelsControls
 from napari.layers import Labels
-from napari.utils.colormaps import colormap_utils
+from napari.utils.colormaps import DirectLabelColormap, colormap_utils
 
 np.random.seed(0)
-_LABELS = np.random.randint(5, size=(10, 15))
-_COLOR = {1: 'white', 2: 'blue', 3: 'green', 4: 'red', 5: 'yellow'}
+_LABELS = np.random.randint(5, size=(10, 15), dtype=np.uint8)
+_COLOR = DirectLabelColormap(
+    color_dict={
+        1: 'white',
+        2: 'blue',
+        3: 'green',
+        4: 'red',
+        5: 'yellow',
+        None: 'black',
+    }
+)
 
 
-@pytest.fixture
-def make_labels_controls(qtbot, color=None):
-    def _make_labels_controls(color=color):
-        layer = Labels(_LABELS, color=color)
+@pytest.fixture()
+def make_labels_controls(qtbot, colormap=None):
+    def _make_labels_controls(colormap=colormap):
+        layer = Labels(_LABELS, colormap=colormap)
         qtctrl = QtLabelsControls(layer)
         qtbot.add_widget(qtctrl)
         return layer, qtctrl
@@ -23,13 +32,25 @@ def make_labels_controls(qtbot, color=None):
 
 def test_changing_layer_color_mode_updates_combo_box(make_labels_controls):
     """Updating layer color mode changes the combo box selection"""
-    layer, qtctrl = make_labels_controls(color=_COLOR)
+    layer, qtctrl = make_labels_controls(colormap=_COLOR)
 
-    original_color_mode = layer.color_mode
-    assert original_color_mode == qtctrl.colorModeComboBox.currentText()
+    assert qtctrl.colorModeComboBox.currentText() == 'direct'
 
-    layer.color_mode = 'auto'
-    assert layer.color_mode == qtctrl.colorModeComboBox.currentText()
+    layer.colormap = layer._random_colormap
+    assert qtctrl.colorModeComboBox.currentText() == 'auto'
+
+
+def test_changing_layer_show_selected_label_updates_check_box(
+    make_labels_controls,
+):
+    """See https://github.com/napari/napari/issues/5371"""
+    layer, qtctrl = make_labels_controls()
+    assert not qtctrl.selectedColorCheckbox.isChecked()
+    assert not layer.show_selected_label
+
+    layer.show_selected_label = True
+
+    assert qtctrl.selectedColorCheckbox.isChecked()
 
 
 def test_rendering_combobox(make_labels_controls):
@@ -47,7 +68,7 @@ def test_rendering_combobox(make_labels_controls):
 
 def test_changing_colormap_updates_colorbox(make_labels_controls):
     """Test that changing the colormap on a layer will update color swatch in the combo box"""
-    layer, qtctrl = make_labels_controls(color=_COLOR)
+    layer, qtctrl = make_labels_controls(colormap=_COLOR)
     color_box = qtctrl.colorBox
 
     layer.selected_label = 1
@@ -102,3 +123,16 @@ def test_preserve_labels_checkbox(make_labels_controls):
     assert not layer.preserve_labels
     qtctrl.preserveLabelsCheckBox.setChecked(True)
     assert layer.preserve_labels
+
+
+def test_change_label_selector_range(make_labels_controls):
+    """Changing the label layer dtype should update label selector range."""
+    layer, qtctrl = make_labels_controls()
+    assert layer.data.dtype == np.uint8
+    assert qtctrl.selectionSpinBox.minimum() == 0
+    assert qtctrl.selectionSpinBox.maximum() == 255
+
+    layer.data = layer.data.astype(np.int8)
+
+    assert qtctrl.selectionSpinBox.minimum() == -128
+    assert qtctrl.selectionSpinBox.maximum() == 127
