@@ -4,7 +4,6 @@ import numpy as np
 from qtpy.QtCore import Qt
 from qtpy.QtGui import QColor, QPainter
 from qtpy.QtWidgets import (
-    QButtonGroup,
     QCheckBox,
     QComboBox,
     QHBoxLayout,
@@ -17,18 +16,16 @@ from superqt import QLargeIntSpinBox
 from napari._qt.layer_controls.qt_layer_controls_base import QtLayerControls
 from napari._qt.utils import set_widgets_enabled_with_opacity
 from napari._qt.widgets._slider_compat import QSlider
-from napari._qt.widgets.qt_mode_buttons import (
-    QtModePushButton,
-    QtModeRadioButton,
-)
+from napari._qt.widgets.qt_mode_buttons import QtModePushButton
 from napari.layers.labels._labels_constants import (
     LABEL_COLOR_MODE_TRANSLATIONS,
+    LabelColorMode,
     LabelsRendering,
     Mode,
 )
 from napari.layers.labels._labels_utils import get_dtype
+from napari.utils import CyclicLabelColormap
 from napari.utils._dtype import get_dtype_limits
-from napari.utils.action_manager import action_manager
 from napari.utils.events import disconnect_events
 from napari.utils.translations import trans
 
@@ -49,6 +46,14 @@ class QtLabelsControls(QtLayerControls):
 
     Attributes
     ----------
+    layer : napari.layers.Labels
+        An instance of a napari Labels layer.
+    MODE : Enum
+        Available modes in the associated layer.
+    PAN_ZOOM_ACTION_NAME : str
+        String id for the pan-zoom action to bind to the pan_zoom button.
+    TRANSFORM_ACTION_NAME : str
+        String id for the transform action to bind to the transform button.
     button_group : qtpy.QtWidgets.QButtonGroup
         Button group of labels layer modes: PAN_ZOOM, PICKER, PAINT, ERASE, or
         FILL.
@@ -56,21 +61,21 @@ class QtLabelsControls(QtLayerControls):
         Button to update colormap of label layer.
     contigCheckBox : qtpy.QtWidgets.QCheckBox
         Checkbox to control if label layer is contiguous.
-    fill_button : qtpy.QtWidgets.QtModeRadioButton
+    fill_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
         Button to select FILL mode on Labels layer.
-    layer : napari.layers.Labels
-        An instance of a napari Labels layer.
     ndimSpinBox : qtpy.QtWidgets.QSpinBox
         Spinbox to control the number of editable dimensions of label layer.
-    paint_button : qtpy.QtWidgets.QtModeRadioButton
+    paint_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
         Button to select PAINT mode on Labels layer.
-    panzoom_button : qtpy.QtWidgets.QtModeRadioButton
+    panzoom_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
         Button to select PAN_ZOOM mode on Labels layer.
-    pick_button : qtpy.QtWidgets.QtModeRadioButton
+    transform_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
+        Button to select TRANSFORM mode on Labels layer.
+    pick_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
         Button to select PICKER mode on Labels layer.
     preserveLabelsCheckBox : qtpy.QtWidgets.QCheckBox
         Checkbox to control if existing labels are preserved
-    erase_button : qtpy.QtWidgets.QtModeRadioButton
+    erase_button : napari._qt.widgets.qt_mode_button.QtModeRadioButton
         Button to select ERASE mode on Labels layer.
     selectionSpinBox : superqt.QLargeIntSpinBox
         Widget to select a specific label by its index.
@@ -86,12 +91,15 @@ class QtLabelsControls(QtLayerControls):
     """
 
     layer: 'napari.layers.Labels'
+    MODE = Mode
+    PAN_ZOOM_ACTION_NAME = 'activate_labels_pan_zoom_mode'
+    TRANSFORM_ACTION_NAME = 'activate_labels_transform_mode'
 
     def __init__(self, layer) -> None:
         super().__init__(layer)
 
-        self.layer.events.mode.connect(self._on_mode_change)
         self.layer.events.rendering.connect(self._on_rendering_change)
+        self.layer.events.colormap.connect(self._on_colormap_change)
         self.layer.events.selected_label.connect(
             self._on_selected_label_change
         )
@@ -101,15 +109,13 @@ class QtLabelsControls(QtLayerControls):
             self._on_n_edit_dimensions_change
         )
         self.layer.events.contour.connect(self._on_contour_change)
-        self.layer.events.editable.connect(self._on_editable_or_visible_change)
-        self.layer.events.visible.connect(self._on_editable_or_visible_change)
         self.layer.events.preserve_labels.connect(
             self._on_preserve_labels_change
         )
         self.layer.events.show_selected_label.connect(
             self._on_show_selected_label_change
         )
-        self.layer.events.color_mode.connect(self._on_color_mode_change)
+        self.layer.events.data.connect(self._on_data_change)
 
         # selection spinbox
         self.selectionSpinBox = QLargeIntSpinBox()
@@ -129,6 +135,15 @@ class QtLabelsControls(QtLayerControls):
         self.brushSizeSlider = sld
         self._on_brush_size_change()
 
+        color_mode_comboBox = QComboBox(self)
+        for data, text in LABEL_COLOR_MODE_TRANSLATIONS.items():
+            data = data.value
+            color_mode_comboBox.addItem(text, data)
+
+        self.colorModeComboBox = color_mode_comboBox
+        self._on_colormap_change()
+        color_mode_comboBox.activated.connect(self.change_color_mode)
+
         contig_cb = QCheckBox()
         contig_cb.setToolTip(trans._('contiguous editing'))
         contig_cb.stateChanged.connect(self.change_contig)
@@ -163,7 +178,7 @@ class QtLabelsControls(QtLayerControls):
 
         selectedColorCheckbox = QCheckBox()
         selectedColorCheckbox.setToolTip(
-            trans._("Display only selected label")
+            trans._('Display only selected label')
         )
         selectedColorCheckbox.stateChanged.connect(self.toggle_selected_mode)
         self.selectedColorCheckbox = selectedColorCheckbox
@@ -177,73 +192,50 @@ class QtLabelsControls(QtLayerControls):
             tooltip=trans._('shuffle colors'),
         )
 
-        self.panzoom_button = QtModeRadioButton(
+        self.pick_button = self._radio_button(
             layer,
-            'pan',
-            Mode.PAN_ZOOM,
-            checked=True,
+            'picker',
+            Mode.PICK,
+            True,
+            'activate_labels_picker_mode',
         )
-        action_manager.bind_button(
-            'napari:activate_labels_pan_zoom_mode', self.panzoom_button
+        self.paint_button = self._radio_button(
+            layer,
+            'paint',
+            Mode.PAINT,
+            True,
+            'activate_labels_paint_mode',
         )
-
-        self.pick_button = QtModeRadioButton(layer, 'picker', Mode.PICK)
-        action_manager.bind_button(
-            'napari:activate_labels_picker_mode', self.pick_button
+        self.polygon_button = self._radio_button(
+            layer,
+            'labels_polygon',
+            Mode.POLYGON,
+            True,
+            'activate_labels_polygon_mode',
         )
-
-        self.paint_button = QtModeRadioButton(layer, 'paint', Mode.PAINT)
-        action_manager.bind_button(
-            'napari:activate_labels_paint_mode', self.paint_button
-        )
-
-        self.fill_button = QtModeRadioButton(
+        self.fill_button = self._radio_button(
             layer,
             'fill',
             Mode.FILL,
+            True,
+            'activate_labels_fill_mode',
         )
-        action_manager.bind_button(
-            'napari:activate_labels_fill_mode',
-            self.fill_button,
-        )
-
-        self.erase_button = QtModeRadioButton(
+        self.erase_button = self._radio_button(
             layer,
             'erase',
             Mode.ERASE,
+            True,
+            'activate_labels_erase_mode',
         )
-        action_manager.bind_button(
-            'napari:activate_labels_erase_mode',
-            self.erase_button,
-        )
-
         # don't bind with action manager as this would remove "Toggle with {shortcut}"
-
-        self._EDIT_BUTTONS = (
-            self.paint_button,
-            self.pick_button,
-            self.fill_button,
-            self.erase_button,
-        )
-
-        self.button_group = QButtonGroup(self)
-        self.button_group.addButton(self.panzoom_button)
-        self.button_group.addButton(self.paint_button)
-        self.button_group.addButton(self.pick_button)
-        self.button_group.addButton(self.fill_button)
-        self.button_group.addButton(self.erase_button)
         self._on_editable_or_visible_change()
 
-        button_row = QHBoxLayout()
-        button_row.addStretch(1)
-        button_row.addWidget(self.colormapUpdate)
-        button_row.addWidget(self.erase_button)
-        button_row.addWidget(self.paint_button)
-        button_row.addWidget(self.fill_button)
-        button_row.addWidget(self.pick_button)
-        button_row.addWidget(self.panzoom_button)
-        button_row.setSpacing(4)
-        button_row.setContentsMargins(0, 0, 0, 5)
+        self.button_grid.addWidget(self.colormapUpdate, 0, 0)
+        self.button_grid.addWidget(self.erase_button, 0, 1)
+        self.button_grid.addWidget(self.paint_button, 0, 2)
+        self.button_grid.addWidget(self.polygon_button, 0, 3)
+        self.button_grid.addWidget(self.fill_button, 0, 4)
+        self.button_grid.addWidget(self.pick_button, 0, 5)
 
         renderComboBox = QComboBox(self)
         rendering_options = [i.value for i in LabelsRendering]
@@ -258,26 +250,12 @@ class QtLabelsControls(QtLayerControls):
 
         self._on_ndisplay_changed()
 
-        color_mode_comboBox = QComboBox(self)
-        for index, (data, text) in enumerate(
-            LABEL_COLOR_MODE_TRANSLATIONS.items()
-        ):
-            data = data.value
-            color_mode_comboBox.addItem(text, data)
-
-            if self.layer.color_mode == data:
-                color_mode_comboBox.setCurrentIndex(index)
-
-        color_mode_comboBox.activated.connect(self.change_color_mode)
-        self.colorModeComboBox = color_mode_comboBox
-        self._on_color_mode_change()
-
         color_layout = QHBoxLayout()
         self.colorBox = QtColorBox(layer)
         color_layout.addWidget(self.colorBox)
         color_layout.addWidget(self.selectionSpinBox)
 
-        self.layout().addRow(button_row)
+        self.layout().addRow(self.button_grid)
         self.layout().addRow(trans._('label:'), color_layout)
         self.layout().addRow(self.opacityLabel, self.opacitySlider)
         self.layout().addRow(trans._('brush size:'), self.brushSizeSlider)
@@ -294,6 +272,13 @@ class QtLabelsControls(QtLayerControls):
             trans._('show\nselected:'), self.selectedColorCheckbox
         )
 
+    def change_color_mode(self):
+        """Change color mode of label layer"""
+        if self.colorModeComboBox.currentData() == LabelColorMode.AUTO.value:
+            self.layer.colormap = self.layer._original_random_colormap
+        else:
+            self.layer.colormap = self.layer._direct_colormap
+
     def _on_mode_change(self, event):
         """Receive layer model mode change event and update checkbox ticks.
 
@@ -305,22 +290,34 @@ class QtLabelsControls(QtLayerControls):
         Raises
         ------
         ValueError
-            Raise error if event.mode is not PAN_ZOOM, PICK, PAINT, ERASE, or
-            FILL
+            Raise error if event.mode is not PAN_ZOOM, PICK, PAINT, ERASE, FILL
+            or TRANSFORM
         """
-        mode = event.mode
-        if mode == Mode.PAN_ZOOM:
-            self.panzoom_button.setChecked(True)
-        elif mode == Mode.PICK:
-            self.pick_button.setChecked(True)
-        elif mode == Mode.PAINT:
-            self.paint_button.setChecked(True)
-        elif mode == Mode.FILL:
-            self.fill_button.setChecked(True)
-        elif mode == Mode.ERASE:
-            self.erase_button.setChecked(True)
-        elif mode != Mode.TRANSFORM:
-            raise ValueError(trans._("Mode not recognized"))
+        super()._on_mode_change(event)
+
+    def _on_colormap_change(self):
+        enable_combobox = not self.layer._is_default_colors(
+            self.layer._direct_colormap.color_dict
+        )
+        self.colorModeComboBox.setEnabled(enable_combobox)
+        if not enable_combobox:
+            self.colorModeComboBox.setToolTip(
+                'Layer needs a user-set DirectLabelColormap to enable direct '
+                'mode.'
+            )
+        if isinstance(self.layer.colormap, CyclicLabelColormap):
+            self.colorModeComboBox.setCurrentIndex(
+                self.colorModeComboBox.findData(LabelColorMode.AUTO.value)
+            )
+        else:
+            self.colorModeComboBox.setCurrentIndex(
+                self.colorModeComboBox.findData(LabelColorMode.DIRECT.value)
+            )
+
+    def _on_data_change(self):
+        """Update label selection spinbox min/max when data changes."""
+        dtype_lims = get_dtype_limits(get_dtype(self.layer))
+        self.selectionSpinBox.setRange(*dtype_lims)
 
     def changeRendering(self, text):
         """Change rendering mode for image display.
@@ -424,10 +421,6 @@ class QtLabelsControls(QtLayerControls):
             Qt.CheckState(state) == Qt.CheckState.Checked
         )
 
-    def change_color_mode(self):
-        """Change color mode of label layer"""
-        self.layer.color_mode = self.colorModeComboBox.currentData()
-
     def _on_contour_change(self):
         """Receive layer model contour value change event and update spinbox."""
         with self.layer.events.contour.blocker():
@@ -454,6 +447,7 @@ class QtLabelsControls(QtLayerControls):
         with self.layer.events.n_edit_dimensions.blocker():
             value = self.layer.n_edit_dimensions
             self.ndimSpinBox.setValue(int(value))
+            self._set_polygon_tool_state()
 
     def _on_contiguous_change(self):
         """Receive layer model contiguous change event and update the checkbox."""
@@ -472,21 +466,6 @@ class QtLabelsControls(QtLayerControls):
                 self.layer.show_selected_label
             )
 
-    def _on_color_mode_change(self):
-        """Receive layer model color."""
-        with self.layer.events.color_mode.blocker():
-            self.colorModeComboBox.setCurrentIndex(
-                self.colorModeComboBox.findData(self.layer.color_mode)
-            )
-
-    def _on_editable_or_visible_change(self):
-        """Receive layer model editable/visible change event & enable/disable buttons."""
-        set_widgets_enabled_with_opacity(
-            self,
-            self._EDIT_BUTTONS,
-            self.layer.editable and self.layer.visible,
-        )
-
     def _on_rendering_change(self):
         """Receive layer model rendering change event and update dropdown menu."""
         with self.layer.events.rendering.blocker():
@@ -495,11 +474,31 @@ class QtLabelsControls(QtLayerControls):
             )
             self.renderComboBox.setCurrentIndex(index)
 
+    def _on_editable_or_visible_change(self):
+        super()._on_editable_or_visible_change()
+        self._set_polygon_tool_state()
+
     def _on_ndisplay_changed(self):
         render_visible = self.ndisplay == 3
         self.renderComboBox.setVisible(render_visible)
         self.renderLabel.setVisible(render_visible)
         self._on_editable_or_visible_change()
+        self._set_polygon_tool_state()
+        super()._on_ndisplay_changed()
+
+    def _set_polygon_tool_state(self):
+        if hasattr(self, 'polygon_button'):
+            set_widgets_enabled_with_opacity(
+                self, [self.polygon_button], self._is_polygon_tool_enabled()
+            )
+
+    def _is_polygon_tool_enabled(self):
+        return (
+            self.layer.editable
+            and self.layer.visible
+            and self.layer.n_edit_dimensions == 2
+            and self.ndisplay == 2
+        )
 
     def deleteLater(self):
         disconnect_events(self.layer.events, self.colorBox)
@@ -511,7 +510,7 @@ class QtColorBox(QWidget):
 
     Parameters
     ----------
-    layer : napari.layers.Layer
+    layer : napari.layers.Labels
         An instance of a napari layer.
     """
 
