@@ -7,7 +7,7 @@ from napari._qt.widgets.qt_extension2reader import Extension2ReaderTable
 from napari.settings import get_settings
 
 
-@pytest.fixture
+@pytest.fixture()
 def extension2reader_widget(qtbot):
     def _extension2reader_widget(**kwargs):
         widget = Extension2ReaderTable(**kwargs)
@@ -19,24 +19,22 @@ def extension2reader_widget(qtbot):
     return _extension2reader_widget
 
 
-@pytest.fixture
+@pytest.fixture()
 def tif_reader(tmp_plugin: DynamicPlugin):
     tmp2 = tmp_plugin.spawn(name='tif_reader', register=True)
 
     @tmp2.contribute.reader(filename_patterns=['*.tif'])
-    def _(path):
-        ...
+    def _(path): ...
 
     return tmp2
 
 
-@pytest.fixture
+@pytest.fixture()
 def npy_reader(tmp_plugin: DynamicPlugin):
     tmp2 = tmp_plugin.spawn(name='npy_reader', register=True)
 
     @tmp2.contribute.reader(filename_patterns=['*.npy'])
-    def _(path):
-        ...
+    def _(path): ...
 
     return tmp2
 
@@ -88,15 +86,14 @@ def test_extension2reader_removal(extension2reader_widget, qtbot):
     qtbot.mouseClick(btn_to_click, Qt.LeftButton)
     assert not get_settings().plugins.extension2reader
     assert widget._table.rowCount() == 1
-    assert "No filename preferences found" in widget._table.item(0, 0).text()
+    assert 'No filename preferences found' in widget._table.item(0, 0).text()
 
 
 def test_all_readers_in_dropdown(
     extension2reader_widget, tmp_plugin, tif_reader
 ):
     @tmp_plugin.contribute.reader(filename_patterns=['*'])
-    def _(path):
-        ...
+    def _(path): ...
 
     npe2_readers = {
         tmp_plugin.name: tmp_plugin.display_name,
@@ -117,8 +114,7 @@ def test_directory_readers_not_in_dropdown(
     @tmp_plugin.contribute.reader(
         filename_patterns=[], accepts_directories=True
     )
-    def f(path):
-        ...
+    def f(path): ...
 
     widget = extension2reader_widget(
         npe2_readers={tmp_plugin.name: tmp_plugin.display_name},
@@ -151,14 +147,25 @@ def test_filtering_readers(
     )
 
 
+@pytest.mark.parametrize('pattern', ['.', '', '/'])
+def test_filtering_readers_problematic_patterns(
+    extension2reader_widget, builtins, tif_reader, npy_reader, pattern
+):
+    widget = extension2reader_widget(
+        npe1_readers={builtins.display_name: builtins.display_name}
+    )
+    widget._filter_compatible_readers(pattern)
+    assert widget._new_reader_dropdown.count() == 1
+    assert widget._new_reader_dropdown.itemText(0) == 'None available'
+
+
 def test_filtering_readers_complex_pattern(
     extension2reader_widget, npy_reader, tif_reader
 ):
     @tif_reader.contribute.reader(
         filename_patterns=['my-specific-folder/*.tif']
     )
-    def f(path):
-        ...
+    def f(path): ...
 
     widget = extension2reader_widget(npe1_readers={})
 
@@ -212,9 +219,8 @@ def test_adding_new_preference_no_asterisk(
 def test_editing_preference(extension2reader_widget, tif_reader):
     tiff2 = tif_reader.spawn(register=True)
 
-    @tiff2.contribute.reader(filename_patterns=["*.tif"])
-    def ff(path):
-        ...
+    @tiff2.contribute.reader(filename_patterns=['*.tif'])
+    def ff(path): ...
 
     get_settings().plugins.extension2reader = {'*.tif': tif_reader.name}
 
