@@ -20,7 +20,7 @@ def make_worker(
     qtbot.addWidget(qtdims)
     nz = 8
     step = 1
-    dims.set_range(0, (0, nz, step))
+    dims.set_range(0, (0, nz - 1, step))
     slider_widget = qtdims.slider_widgets[0]
     slider_widget.loop_mode = loop_mode
     slider_widget.fps = fps
@@ -34,7 +34,7 @@ def make_worker(
         if worker._count < nframes:
             worker._count += 1
         else:
-            worker.finish()
+            worker._stop()
 
     def count_reached():
         assert worker._count >= nframes
@@ -42,6 +42,7 @@ def make_worker(
     def go():
         worker.work()
         qtbot.waitUntil(count_reached, timeout=6000)
+        worker._stop()
         return worker.current
 
     worker.frame_requested.connect(bump)
@@ -68,7 +69,9 @@ CONDITIONS = [
 ]
 
 
-@pytest.mark.parametrize("nframes,fps,mode,rng,result", CONDITIONS)
+@pytest.mark.parametrize(
+    ('nframes', 'fps', 'mode', 'rng', 'result'), CONDITIONS
+)
 def test_animation_thread_variants(qtbot, nframes, fps, mode, rng, result):
     """This is mostly testing that AnimationWorker.advance works as expected"""
     with make_worker(
@@ -78,12 +81,11 @@ def test_animation_thread_variants(qtbot, nframes, fps, mode, rng, result):
     if rng:
         nrange = rng[1] - rng[0] + 1
         expected = rng[0] + result(nframes, nrange)
-        assert expected - 1 <= current <= expected + 1
     else:
         expected = result(nframes, worker.nz)
         # assert current == expected
         # relaxing for CI OSX tests
-        assert expected - 1 <= current <= expected + 1
+    assert expected - 1 <= current <= expected + 1
 
 
 def test_animation_thread_once(qtbot):
@@ -122,56 +124,19 @@ def test_play_raises_index_errors(qtbot, ref_view):
     # play axis is out of range
     with pytest.raises(IndexError):
         view.dims.play(4, 20)
-        qtbot.wait(20)
-        view.dims.stop()
 
     # data doesn't have 20 frames
     with pytest.raises(IndexError):
         view.dims.play(0, 20, frame_range=[2, 20])
-        qtbot.wait(20)
-        view.dims.stop()
 
 
 def test_play_raises_value_errors(qtbot, ref_view):
     view = ref_view()
-    # frame_range[1] not > frame_range[0]
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match='must be <='):
         view.dims.play(0, 20, frame_range=[2, 2])
-        qtbot.wait(20)
-        view.dims.stop()
 
-    # that's not a valid loop_mode
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match='loop_mode must be one of'):
         view.dims.play(0, 20, loop_mode=5)
-        qtbot.wait(20)
-        view.dims.stop()
-
-
-@pytest.mark.skip(reason="fails too often... tested indirectly elsewhere")
-def test_play_api(qtbot, ref_view):
-    """Test that the QtDims.play() function advances a few frames"""
-    view = ref_view()
-    view.dims._frame = 0
-
-    def increment():
-        view.dims._frame += 1
-        # if we don't "enable play" again, view.dims won't request a new frame
-        view.dims._play_ready = True
-
-    view.dims.dims.events.current_step.connect(increment)
-
-    view.dims.play(0, 20)
-    # wait for the thread to start before timing...
-    qtbot.waitSignal(view.dims._animation_thread.started, timeout=10000)
-    qtbot.wait(370)
-    with qtbot.waitSignal(view.dims._animation_thread.finished, timeout=7000):
-        view.dims.stop()
-    A = view.dims._frame
-    assert A >= 3
-
-    # make sure the stop button actually worked
-    qtbot.wait(150)
-    assert A == view.dims._frame
 
 
 def test_playing_hidden_slider_does_nothing(ref_view):
