@@ -1,10 +1,10 @@
 import warnings
-from typing import Optional, Tuple
+from typing import Optional
 
 import numpy as np
 from qtpy.QtCore import Slot
 from qtpy.QtGui import QFont, QFontMetrics
-from qtpy.QtWidgets import QLineEdit, QSizePolicy, QVBoxLayout, QWidget
+from qtpy.QtWidgets import QSizePolicy, QVBoxLayout, QWidget
 
 from napari._qt.widgets.qt_dims_slider import QtDimSliderWidget
 from napari.components.dims import Dims
@@ -44,7 +44,6 @@ class QtDims(QWidget):
         # True / False if slider is or is not displayed
         self._displayed_sliders = []
 
-        self._play_ready = True  # False if currently awaiting a draw event
         self._animation_thread = None
         self._animation_worker = None
 
@@ -99,7 +98,6 @@ class QtDims(QWidget):
 
     def _update_display(self):
         """Updates display for all sliders."""
-        self.stop()
         widgets = reversed(list(enumerate(self.slider_widgets)))
         nsteps = self.dims.nsteps
         for axis, widget in widgets:
@@ -117,10 +115,10 @@ class QtDims(QWidget):
         self.setMinimumHeight(nsliders * self.SLIDERHEIGHT)
         self._resize_slice_labels()
         self._resize_axis_labels()
+        self.stop()
 
     def _update_nsliders(self):
         """Updates the number of sliders based on the number of dimensions."""
-        self.stop()
         self._trim_sliders(0)
         self._create_sliders(self.dims.ndim)
         self._update_display()
@@ -128,6 +126,7 @@ class QtDims(QWidget):
             self._update_range()
             if self._displayed_sliders[i]:
                 self._update_slider()
+        self.stop()
 
     def _resize_axis_labels(self):
         """When any of the labels get updated, this method updates all label
@@ -141,11 +140,10 @@ class QtDims(QWidget):
             if displayed
         ]
         if displayed_labels:
-            fm = QFontMetrics(QFont("", 0))
-            labels = self.findChildren(QLineEdit, 'axis_label')
+            fm = self.fontMetrics()
             # set maximum width to no more than 20% of slider width
             maxwidth = int(self.slider_widgets[0].width() * 0.2)
-            # set new base width to the width of the longest label being displayed
+            # set new width to the width of the longest label being displayed
             newwidth = max(
                 [
                     int(fm.boundingRect(dlab.text()).width())
@@ -153,8 +151,14 @@ class QtDims(QWidget):
                 ]
             )
 
-            for labl in labels:
-                labl_width = min([newwidth + 10, maxwidth])
+            for slider in self.slider_widgets:
+                labl = slider.axis_label
+                # here the average width of a character is used as base measure
+                # to add some extra width. We use 4 to take into account a
+                # space and the possible 3 dots (`...`) for elided text
+                margin_width = int(fm.averageCharWidth() * 4)
+                base_labl_width = min([newwidth, maxwidth])
+                labl_width = base_labl_width + margin_width
                 labl.setFixedWidth(labl_width)
 
     def _resize_slice_labels(self):
@@ -170,8 +174,8 @@ class QtDims(QWidget):
                 if length > width:
                     width = length
         # gui width of a string of length `width`
-        fm = QFontMetrics(QFont("", 0))
-        width = fm.boundingRect("8" * width).width()
+        fm = QFontMetrics(QFont('', 0))
+        width = fm.boundingRect('8' * width).width()
         for labl in self.findChildren(QWidget, 'slice_label'):
             labl.setFixedWidth(width + 6)
 
@@ -191,6 +195,7 @@ class QtDims(QWidget):
             slider_widget.axis_label.textChanged.connect(
                 self._resize_axis_labels
             )
+            slider_widget.size_changed.connect(self._resize_axis_labels)
             slider_widget.play_button.play_requested.connect(self.play)
             self.layout().addWidget(slider_widget)
             self.slider_widgets.insert(0, slider_widget)
@@ -238,7 +243,7 @@ class QtDims(QWidget):
         axis: int = 0,
         fps: Optional[float] = None,
         loop_mode: Optional[str] = None,
-        frame_range: Optional[Tuple[int, int]] = None,
+        frame_range: Optional[tuple[int, int]] = None,
     ):
         """Animate (play) axis.
 
@@ -324,7 +329,7 @@ class QtDims(QWidget):
     def cleaned_worker(self):
         self._animation_thread = None
         self._animation_worker = None
-        self.enable_play()
+        self.dims._play_ready = True
 
     @property
     def is_playing(self):
@@ -335,8 +340,8 @@ class QtDims(QWidget):
             )
         except RuntimeError as e:  # pragma: no cover
             if (
-                "wrapped C/C++ object of type" not in e.args[0]
-                and "Internal C++ object" not in e.args[0]
+                'wrapped C/C++ object of type' not in e.args[0]
+                and 'Internal C++ object' not in e.args[0]
             ):
                 # checking if threat is partially deleted. Otherwise
                 # reraise exception. For more details see:
@@ -352,16 +357,11 @@ class QtDims(QWidget):
         the canvas can draw, this will drop the intermediate frames, keeping
         the effective frame rate constant even if the canvas cannot keep up.
         """
-        if self._play_ready:
+        if self.dims._play_ready:
             # disable additional point advance requests until this one draws
-            self._play_ready = False
+            self.dims._play_ready = False
             self.dims.set_current_step(axis, frame)
 
-    def enable_play(self, *args):
-        # this is mostly here to connect to the main SceneCanvas.events.draw
-        # event in the qt_viewer
-        self._play_ready = True
-
     def closeEvent(self, event):
         [w.deleteLater() for w in self.slider_widgets]
         self.deleteLater()
