@@ -8,7 +8,7 @@ def test_viewer_key_bindings(make_napari_viewer):
     """Test adding key bindings to the viewer"""
     np.random.seed(0)
     viewer = make_napari_viewer()
-    view = viewer.window.qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
 
     mock_press = Mock()
     mock_release = Mock()
@@ -40,7 +40,7 @@ def test_viewer_key_bindings(make_napari_viewer):
         mock_shift_release.method()
 
     # Simulate press only
-    view.canvas.events.key_press(key=keys.Key('F'))
+    canvas._scene_canvas.events.key_press(key=keys.Key('F'))
     mock_press.method.assert_called_once()
     mock_press.reset_mock()
     mock_release.method.assert_not_called()
@@ -48,7 +48,7 @@ def test_viewer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.key_release(key=keys.Key('F'))
+    canvas._scene_canvas.events.key_release(key=keys.Key('F'))
     mock_press.method.assert_not_called()
     mock_release.method.assert_called_once()
     mock_release.reset_mock()
@@ -56,7 +56,9 @@ def test_viewer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate press only
-    view.canvas.events.key_press(key=keys.Key('F'), modifiers=[keys.SHIFT])
+    canvas._scene_canvas.events.key_press(
+        key=keys.Key('F'), modifiers=[keys.SHIFT]
+    )
     mock_press.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_shift_press.method.assert_called_once()
@@ -64,7 +66,9 @@ def test_viewer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.key_release(key=keys.Key('F'), modifiers=[keys.SHIFT])
+    canvas._scene_canvas.events.key_release(
+        key=keys.Key('F'), modifiers=[keys.SHIFT]
+    )
     mock_press.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_shift_press.method.assert_not_called()
@@ -76,7 +80,7 @@ def test_layer_key_bindings(make_napari_viewer):
     """Test adding key bindings to a layer"""
     np.random.seed(0)
     viewer = make_napari_viewer()
-    view = viewer.window.qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
 
     layer = viewer.add_image(np.random.random((10, 20)))
     viewer.layers.selection.add(layer)
@@ -108,7 +112,7 @@ def test_layer_key_bindings(make_napari_viewer):
         mock_shift_release.method()
 
     # Simulate press only
-    view.canvas.events.key_press(key=keys.Key('F'))
+    canvas._scene_canvas.events.key_press(key=keys.Key('F'))
     mock_press.method.assert_called_once()
     mock_press.reset_mock()
     mock_release.method.assert_not_called()
@@ -116,7 +120,7 @@ def test_layer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.key_release(key=keys.Key('F'))
+    canvas._scene_canvas.events.key_release(key=keys.Key('F'))
     mock_press.method.assert_not_called()
     mock_release.method.assert_called_once()
     mock_release.reset_mock()
@@ -124,7 +128,9 @@ def test_layer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate press only
-    view.canvas.events.key_press(key=keys.Key('F'), modifiers=[keys.SHIFT])
+    canvas._scene_canvas.events.key_press(
+        key=keys.Key('F'), modifiers=[keys.SHIFT]
+    )
     mock_press.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_shift_press.method.assert_called_once()
@@ -132,7 +138,9 @@ def test_layer_key_bindings(make_napari_viewer):
     mock_shift_release.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.key_release(key=keys.Key('F'), modifiers=[keys.SHIFT])
+    canvas._scene_canvas.events.key_release(
+        key=keys.Key('F'), modifiers=[keys.SHIFT]
+    )
     mock_press.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_shift_press.method.assert_not_called()
@@ -143,12 +151,12 @@ def test_layer_key_bindings(make_napari_viewer):
 def test_reset_scroll_progress(make_napari_viewer):
     """Test select all key binding."""
     viewer = make_napari_viewer()
-    view = viewer.window.qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
     assert viewer.dims._scroll_progress == 0
 
-    view.canvas.events.key_press(key=keys.Key('Control'))
+    canvas._scene_canvas.events.key_press(key=keys.Key('Control'))
     viewer.dims._scroll_progress = 10
     assert viewer.dims._scroll_progress == 10
 
-    view.canvas.events.key_release(key=keys.Key('Control'))
+    canvas._scene_canvas.events.key_release(key=keys.Key('Control'))
     assert viewer.dims._scroll_progress == 0
