@@ -4,17 +4,16 @@ from unittest.mock import Mock
 import numpy as np
 
 from napari._tests.utils import skip_on_win_ci
-from napari.layers.base._base_mouse_bindings import InteractionBoxHandle
 
 
 @skip_on_win_ci
-def test_viewer_mouse_bindings(make_napari_viewer):
+def test_viewer_mouse_bindings(qtbot, make_napari_viewer):
     """Test adding mouse bindings to the viewer"""
     np.random.seed(0)
     viewer = make_napari_viewer()
-    view = viewer.window._qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
 
-    if os.getenv("CI"):
+    if os.getenv('CI'):
         viewer.show()
 
     mock_press = Mock()
@@ -46,7 +45,7 @@ def test_viewer_mouse_bindings(make_napari_viewer):
         mock_move.method()
 
     # Simulate press only
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
     mock_press.method.assert_called_once()
     mock_press.reset_mock()
     mock_drag.method.assert_not_called()
@@ -54,7 +53,9 @@ def test_viewer_mouse_bindings(make_napari_viewer):
     mock_move.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_called_once()
@@ -62,7 +63,7 @@ def test_viewer_mouse_bindings(make_napari_viewer):
     mock_move.method.assert_not_called()
 
     # Simulate move with no press
-    view.canvas.events.mouse_move(pos=(0, 0), modifiers=())
+    canvas._scene_canvas.events.mouse_move(pos=(0, 0), modifiers=())
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
@@ -70,11 +71,16 @@ def test_viewer_mouse_bindings(make_napari_viewer):
     mock_move.reset_mock()
 
     # Simulate press, drag, release
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
-    view.canvas.events.mouse_move(
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_move(
         pos=(0, 0), modifiers=(), button=0, press_event=True
     )
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
+    qtbot.wait(10)
     mock_press.method.assert_called_once()
     mock_drag.method.assert_called_once()
     mock_release.method.assert_called_once()
@@ -82,13 +88,13 @@ def test_viewer_mouse_bindings(make_napari_viewer):
 
 
 @skip_on_win_ci
-def test_layer_mouse_bindings(make_napari_viewer):
+def test_layer_mouse_bindings(qtbot, make_napari_viewer):
     """Test adding mouse bindings to a layer that is selected"""
     np.random.seed(0)
     viewer = make_napari_viewer()
-    view = viewer.window._qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
 
-    if os.getenv("CI"):
+    if os.getenv('CI'):
         viewer.show()
 
     layer = viewer.add_image(np.random.random((10, 20)))
@@ -122,7 +128,7 @@ def test_layer_mouse_bindings(make_napari_viewer):
         mock_move.method()
 
     # Simulate press only
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
     mock_press.method.assert_called_once()
     mock_press.reset_mock()
     mock_drag.method.assert_not_called()
@@ -130,7 +136,9 @@ def test_layer_mouse_bindings(make_napari_viewer):
     mock_move.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_called_once()
@@ -138,7 +146,7 @@ def test_layer_mouse_bindings(make_napari_viewer):
     mock_move.method.assert_not_called()
 
     # Simulate move with no press
-    view.canvas.events.mouse_move(pos=(0, 0), modifiers=())
+    canvas._scene_canvas.events.mouse_move(pos=(0, 0), modifiers=())
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
@@ -146,43 +154,30 @@ def test_layer_mouse_bindings(make_napari_viewer):
     mock_move.reset_mock()
 
     # Simulate press, drag, release
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
-    view.canvas.events.mouse_move(
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_move(
         pos=(0, 0), modifiers=(), button=0, press_event=True
     )
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
+    qtbot.wait(10)
     mock_press.method.assert_called_once()
     mock_drag.method.assert_called_once()
     mock_release.method.assert_called_once()
     mock_move.method.assert_not_called()
-    mock_press.reset_mock()
-    mock_drag.reset_mock()
-    mock_release.reset_mock()
-
-    # simulate hover in transform mode
-    layer.mode = 'transform'
-    # go to middle of the canvas, so we're sure to be inside
-    position = tuple(d // 2 for d in view.canvas.size)
-    view.canvas.events.mouse_move(pos=position, modifiers=())
-    mock_press.method.assert_not_called()
-    mock_drag.method.assert_not_called()
-    mock_release.method.assert_not_called()
-    mock_move.method.assert_called_once()
-    assert (
-        layer._overlays['transform_box'].selected_handle
-        == InteractionBoxHandle.INSIDE
-    )
-    mock_move.reset_mock()
 
 
 @skip_on_win_ci
-def test_unselected_layer_mouse_bindings(make_napari_viewer):
+def test_unselected_layer_mouse_bindings(qtbot, make_napari_viewer):
     """Test adding mouse bindings to a layer that is not selected"""
     np.random.seed(0)
     viewer = make_napari_viewer()
-    view = viewer.window._qt_viewer
+    canvas = viewer.window._qt_viewer.canvas
 
-    if os.getenv("CI"):
+    if os.getenv('CI'):
         viewer.show()
 
     layer = viewer.add_image(np.random.random((10, 20)))
@@ -216,32 +211,39 @@ def test_unselected_layer_mouse_bindings(make_napari_viewer):
         mock_move.method()
 
     # Simulate press only
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_move.method.assert_not_called()
 
     # Simulate release only
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_move.method.assert_not_called()
 
     # Simulate move with no press
-    view.canvas.events.mouse_move(pos=(0, 0), modifiers=())
+    canvas._scene_canvas.events.mouse_move(pos=(0, 0), modifiers=())
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
     mock_move.method.assert_not_called()
 
     # Simulate press, drag, release
-    view.canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
-    view.canvas.events.mouse_move(
+    canvas._scene_canvas.events.mouse_press(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_move(
         pos=(0, 0), modifiers=(), button=0, press_event=True
     )
-    view.canvas.events.mouse_release(pos=(0, 0), modifiers=(), button=0)
+    qtbot.wait(10)
+    canvas._scene_canvas.events.mouse_release(
+        pos=(0, 0), modifiers=(), button=0
+    )
+    qtbot.wait(10)
     mock_press.method.assert_not_called()
     mock_drag.method.assert_not_called()
     mock_release.method.assert_not_called()
