@@ -1,10 +1,8 @@
-import collections
-
 import numpy as np
 import pytest
 
 from napari._tests.utils import skip_local_popups, skip_on_win_ci
-from napari.utils._proxies import ReadOnlyWrapper
+from napari.utils._test_utils import read_only_mouse_event
 from napari.utils.interactions import (
     mouse_move_callbacks,
     mouse_press_callbacks,
@@ -370,48 +368,36 @@ def test_labels_painting(make_napari_viewer):
     layer.selected_label = 3
 
     # Simulate click
-    Event = collections.namedtuple(
-        'Event', field_names=['type', 'is_dragging', 'position']
-    )
-
-    # Simulate click
-    event = ReadOnlyWrapper(
-        Event(
-            type='mouse_press',
-            is_dragging=False,
-            position=viewer.cursor.position,
-        )
+    event = read_only_mouse_event(
+        type='mouse_press',
+        is_dragging=False,
+        position=viewer.cursor.position,
     )
     mouse_press_callbacks(layer, event)
 
     viewer.cursor.position = (100, 100)
 
     # Simulate drag
-    event = ReadOnlyWrapper(
-        Event(
-            type='mouse_move',
-            is_dragging=True,
-            position=viewer.cursor.position,
-        )
+    event = read_only_mouse_event(
+        type='mouse_move',
+        is_dragging=True,
+        position=viewer.cursor.position,
     )
     mouse_move_callbacks(layer, event)
 
     # Simulate release
-    event = ReadOnlyWrapper(
-        Event(
-            type='mouse_release',
-            is_dragging=False,
-            position=viewer.cursor.position,
-        )
+    event = read_only_mouse_event(
+        type='mouse_release',
+        is_dragging=False,
+        position=viewer.cursor.position,
     )
+
     mouse_release_callbacks(layer, event)
 
-    event = ReadOnlyWrapper(
-        Event(
-            type='mouse_press',
-            is_dragging=False,
-            position=viewer.cursor.position,
-        )
+    event = read_only_mouse_event(
+        type='mouse_press',
+        is_dragging=False,
+        position=viewer.cursor.position,
     )
     mouse_press_callbacks(layer, event)
 
@@ -421,7 +407,7 @@ def test_labels_painting(make_napari_viewer):
     assert screenshot[:, :, :2].max() > 0
 
 
-@pytest.mark.skip("Welcome visual temporarily disabled")
+@pytest.mark.skip('Welcome visual temporarily disabled')
 @skip_on_win_ci
 @skip_local_popups
 def test_welcome(make_napari_viewer):
@@ -522,8 +508,7 @@ def test_blending_modes_with_canvas(make_napari_viewer):
     img1_layer = viewer.add_image(img1)
     img2_layer = viewer.add_image(img2)
 
-    # invert shape cause vispy xy is flipped
-    viewer.window._qt_viewer.canvas.size = shape[::-1]
+    viewer.window._qt_viewer.canvas.size = shape
     viewer.camera.zoom = 1
 
     # check that additive behaves correctly with black canvas
