@@ -1,14 +1,17 @@
 from __future__ import annotations
 
 import weakref
+from collections.abc import Generator
 from contextlib import contextmanager
 from contextvars import ContextVar
-from typing import Optional, Tuple
+from typing import Any, Optional
+from weakref import ReferenceType
 
 from magicgui.widgets import FunctionGui
-from pydantic import BaseModel, validator
+from typing_extensions import Self
 
-from .base.base import Layer
+from napari._pydantic_compat import BaseModel, validator
+from napari.layers.base.base import Layer
 
 
 class Source(BaseModel):
@@ -31,7 +34,7 @@ class Source(BaseModel):
 
     path: Optional[str] = None
     reader_plugin: Optional[str] = None
-    sample: Optional[Tuple[str, str]] = None
+    sample: Optional[tuple[str, str]] = None
     widget: Optional[FunctionGui] = None
     parent: Optional[Layer] = None
 
@@ -39,11 +42,11 @@ class Source(BaseModel):
         arbitrary_types_allowed = True
         frozen = True
 
-    @validator('parent')
-    def make_weakref(cls, layer: Layer):
+    @validator('parent', allow_reuse=True)
+    def make_weakref(cls, layer: Layer) -> ReferenceType[Layer]:
         return weakref.ref(layer)
 
-    def __deepcopy__(self, memo):
+    def __deepcopy__(self, memo: Any) -> Self:
         """Custom deepcopy implementation.
 
         this prevents deep copy. `Source` doesn't really need to be copied
@@ -59,7 +62,7 @@ _LAYER_SOURCE: ContextVar[dict] = ContextVar('_LAYER_SOURCE', default={})
 
 
 @contextmanager
-def layer_source(**source_kwargs):
+def layer_source(**source_kwargs: Any) -> Generator[None, None, None]:
     """Creates context in which all layers will be given `source_kwargs`.
 
     The module-level variable `_LAYER_SOURCE` holds a set of key-value pairs
@@ -101,7 +104,7 @@ def layer_source(**source_kwargs):
         _LAYER_SOURCE.reset(token)
 
 
-def current_source():
+def current_source() -> Source:
     """Get the current layer :class:`Source` (inferred from context).
 
     The main place this function is used is in :meth:`Layer.__init__`.
