@@ -1,33 +1,42 @@
 import warnings
 from collections import deque
+from collections.abc import Sequence
 from contextlib import contextmanager
 from typing import (
-    Dict,
-    List,
+    Any,
+    Callable,
+    ClassVar,
     Optional,
-    Sequence,
-    Tuple,
     Union,
     cast,
 )
 
 import numpy as np
+import numpy.typing as npt
 import pandas as pd
 from scipy import ndimage as ndi
+from skimage.draw import polygon2mask
 
+from napari.layers._data_protocols import LayerDataProtocol
+from napari.layers._multiscale_data import MultiScaleData
+from napari.layers._scalar_field.scalar_field import ScalarFieldBase
 from napari.layers.base import Layer, no_op
 from napari.layers.base._base_mouse_bindings import (
     highlight_box_handles,
     transform_with_box,
 )
 from napari.layers.image._image_utils import guess_multiscale
-from napari.layers.image.image import _ImageBase
+from napari.layers.image._slice import _ImageSliceResponse
 from napari.layers.labels._labels_constants import (
     LabelColorMode,
     LabelsRendering,
     Mode,
 )
-from napari.layers.labels._labels_mouse_bindings import draw, pick
+from napari.layers.labels._labels_mouse_bindings import (
+    BrushSizeOnMouseMove,
+    draw,
+    pick,
+)
 from napari.layers.labels._labels_utils import (
     expand_slice,
     get_contours,
@@ -36,7 +45,6 @@ from napari.layers.labels._labels_utils import (
     sphere_indices,
 )
 from napari.layers.utils.layer_utils import _FeatureTable
-from napari.utils import config
 from napari.utils._dtype import normalize_dtype, vispy_texture_dtype
 from napari.utils._indexing import elements_in_slice, index_in_slice
 from napari.utils.colormaps import (
@@ -45,22 +53,22 @@ from napari.utils.colormaps import (
 )
 from napari.utils.colormaps.colormap import (
     CyclicLabelColormap,
-    DirectLabelColormap,
     LabelColormapBase,
+    _normalize_label_colormap,
 )
 from napari.utils.colormaps.colormap_utils import shuffle_and_extend_colormap
 from napari.utils.events import EmitterGroup, Event
 from napari.utils.events.custom_types import Array
-from napari.utils.events.event import WarningEmitter
 from napari.utils.geometry import clamp_point_to_bounding_box
-from napari.utils.migrations import deprecated_constructor_arg_by_attr
-from napari.utils.misc import _is_array_type
+from napari.utils.misc import StringEnum, _is_array_type
 from napari.utils.naming import magic_name
 from napari.utils.status_messages import generate_layer_coords_status
 from napari.utils.translations import trans
 
+__all__ = ('Labels',)
 
-class Labels(_ImageBase):
+
+class Labels(ScalarFieldBase):
     """Labels (or segmentation) layer.
 
     An image-like layer where every pixel contains an integer ID
@@ -78,6 +86,9 @@ class Labels(_ImageBase):
         the final column is a length N translation vector and a 1 or a napari
         `Affine` transform object. Applied as an extra transform on top of the
         provided scale, rotate, and shear values.
+    axis_labels : tuple of str, optional
+        Dimension names of the layer data.
+        If not provided, axis_labels will be set to (..., 'axis -2', 'axis -1').
     blending : str
         One of a list of preset blending modes that determines how RGB and
         alpha values of the layer visual get mixed. Allowed values are
@@ -116,6 +127,9 @@ class Labels(_ImageBase):
         Properties defining plane rendering in 3D. Properties are defined in
         data coordinates. Valid dictionary keys are
         {'position', 'normal', 'thickness', and 'enabled'}.
+    projection_mode : str
+        How data outside the viewed dimensions but inside the thick Dims slice will
+        be projected onto the viewed dimensions
     properties : dict {str: array (N,)} or DataFrame
         Properties for each label. Each property should be an array of length
         N, where N is the number of labels, and the first property corresponds
@@ -138,6 +152,9 @@ class Labels(_ImageBase):
         ones along the main diagonal.
     translate : tuple of float
         Translation values for the layer.
+    units : tuple of str or pint.Unit, optional
+        Units of the layer data in world coordinates.
+        If not provided, the default units are assumed to be pixels.
     visible : bool
         Whether the layer visual is currently being displayed.
 
@@ -149,6 +166,8 @@ class Labels(_ImageBase):
         belongs to. The label 0 is rendered as transparent. Please note
         multiscale rendering is only supported in 2D. In 3D, only
         the lowest resolution scale is displayed.
+    axis_labels : tuple of str
+        Dimension names of the layer data.
     multiscale : bool
         Whether the data is a multiscale image or not. Multiscale data is
         represented by a list of array like image data. The first image in the
@@ -216,6 +235,8 @@ class Labels(_ImageBase):
         Properties defining plane rendering in 3D.
     experimental_clipping_planes : ClippingPlaneList
         Clipping planes defined in data coordinates, used to clip the volume.
+    units: tuple of pint.Unit
+        Units of the layer data in world coordinates.
 
     Notes
     -----
@@ -229,43 +250,48 @@ class Labels(_ImageBase):
 
     _modeclass = Mode
 
-    _drag_modes = {
+    _drag_modes: ClassVar[dict[Mode, Callable[['Labels', Event], None]]] = {  # type: ignore[assignment]
         Mode.PAN_ZOOM: no_op,
         Mode.TRANSFORM: transform_with_box,
         Mode.PICK: pick,
         Mode.PAINT: draw,
         Mode.FILL: draw,
         Mode.ERASE: draw,
+        Mode.POLYGON: no_op,  # the overlay handles mouse events in this mode
     }
 
-    _move_modes = {
+    brush_size_on_mouse_move = BrushSizeOnMouseMove(min_brush_size=1)
+
+    _move_modes: ClassVar[
+        dict[StringEnum, Callable[['Labels', Event], None]]
+    ] = {  # type: ignore[assignment]
         Mode.PAN_ZOOM: no_op,
         Mode.TRANSFORM: highlight_box_handles,
         Mode.PICK: no_op,
-        Mode.PAINT: no_op,
+        Mode.PAINT: brush_size_on_mouse_move,
         Mode.FILL: no_op,
-        Mode.ERASE: no_op,
+        Mode.ERASE: brush_size_on_mouse_move,
+        Mode.POLYGON: no_op,  # the overlay handles mouse events in this mode
     }
 
-    _cursor_modes = {
+    _cursor_modes: ClassVar[dict[Mode, str]] = {  # type: ignore[assignment]
         Mode.PAN_ZOOM: 'standard',
         Mode.TRANSFORM: 'standard',
         Mode.PICK: 'cross',
         Mode.PAINT: 'circle',
         Mode.FILL: 'cross',
         Mode.ERASE: 'circle',
+        Mode.POLYGON: 'cross',
     }
 
     _history_limit = 100
 
-    @deprecated_constructor_arg_by_attr("color")
-    @deprecated_constructor_arg_by_attr("num_colors")
-    @deprecated_constructor_arg_by_attr("seed")
     def __init__(
         self,
         data,
         *,
         affine=None,
+        axis_labels=None,
         blending='translucent',
         cache=True,
         colormap=None,
@@ -277,12 +303,14 @@ class Labels(_ImageBase):
         name=None,
         opacity=0.7,
         plane=None,
+        projection_mode='none',
         properties=None,
         rendering='iso_categorical',
         rotate=None,
         scale=None,
         shear=None,
         translate=None,
+        units=None,
         visible=True,
     ) -> None:
         if name is None and data is not None:
@@ -307,36 +335,30 @@ class Labels(_ImageBase):
 
         super().__init__(
             data,
-            rendering=rendering,
-            depiction=depiction,
-            name=name,
-            metadata=metadata,
-            scale=scale,
-            translate=translate,
-            rotate=rotate,
-            shear=shear,
             affine=affine,
-            opacity=opacity,
+            axis_labels=axis_labels,
             blending=blending,
-            visible=visible,
-            multiscale=multiscale,
             cache=cache,
-            plane=plane,
+            depiction=depiction,
             experimental_clipping_planes=experimental_clipping_planes,
+            rendering=rendering,
+            metadata=metadata,
+            multiscale=multiscale,
+            name=name,
+            scale=scale,
+            shear=shear,
+            plane=plane,
+            opacity=opacity,
+            projection_mode=projection_mode,
+            rotate=rotate,
+            translate=translate,
+            units=units,
+            visible=visible,
         )
 
         self.events.add(
             brush_shape=Event,
             brush_size=Event,
-            color_mode=WarningEmitter(
-                trans._(
-                    'Labels.events.color_mode is deprecated since 0.4.19 and '
-                    'will be removed in 0.5.0, please use '
-                    'Labels.events.colormap.',
-                    deferred=True,
-                ),
-                type_name='color_mode',
-            ),
             colormap=Event,
             contiguous=Event,
             contour=Event,
@@ -350,6 +372,12 @@ class Labels(_ImageBase):
             show_selected_label=Event,
         )
 
+        from napari.components.overlays.labels_polygon import (
+            LabelsPolygonOverlay,
+        )
+
+        self._overlays.update({'polygon': LabelsPolygonOverlay()})
+
         self._feature_table = _FeatureTable.from_layer(
             features=features, properties=properties
         )
@@ -362,6 +390,7 @@ class Labels(_ImageBase):
         self._selected_label = 1
         self.colormap.selection = self._selected_label
         self.colormap.use_selection = self._show_selected_label
+        self._prev_selected_label = None
         self._selected_color = self.get_color(self._selected_label)
         self._updated_slice = None
         if colormap is not None:
@@ -372,7 +401,6 @@ class Labels(_ImageBase):
 
     def _post_init(self):
         self._reset_history()
-
         # Trigger generation of view slice and thumbnail
         self.refresh()
         self._reset_editable()
@@ -430,7 +458,7 @@ class Labels(_ImageBase):
     @contour.setter
     def contour(self, contour: int) -> None:
         if contour < 0:
-            raise ValueError("contour value must be >= 0")
+            raise ValueError('contour value must be >= 0')
         self._contour = int(contour)
         self.events.contour()
         self.refresh()
@@ -455,33 +483,6 @@ class Labels(_ImageBase):
         )
         return abs(self.brush_size * min_scale)
 
-    @property
-    def seed(self):
-        """float: Seed for colormap random generator."""
-        warnings.warn(
-            "seed is deprecated since 0.4.19 and will be removed in 0.5.0, "
-            "please check Labels.colormap directly.",
-            FutureWarning,
-            stacklevel=2,
-        )
-        return self._random_colormap.seed
-
-    @seed.setter
-    def seed(self, seed):
-        warnings.warn(
-            "seed is deprecated since 0.4.19 and will be removed in 0.5.0, "
-            "please use the new_colormap method instead, or set the colormap "
-            "directly.",
-            FutureWarning,
-            stacklevel=2,
-        )
-
-        self.colormap = label_colormap(
-            len(self.colormap) - 1,
-            seed=seed,
-            background_value=self.colormap.background_value,
-        )
-
     def new_colormap(self, seed: Optional[int] = None):
         if seed is None:
             seed = np.random.default_rng().integers(2**32 - 1)
@@ -501,6 +502,7 @@ class Labels(_ImageBase):
         self._set_colormap(colormap)
 
     def _set_colormap(self, colormap):
+        colormap = _normalize_label_colormap(colormap)
         if isinstance(colormap, CyclicLabelColormap):
             self._random_colormap = colormap
             self._original_random_colormap = colormap
@@ -522,6 +524,7 @@ class Labels(_ImageBase):
             else:
                 color_mode = LabelColorMode.DIRECT
                 self._colormap = self._direct_colormap
+        self._cached_labels = None  # invalidate the cached color mapping
         self._selected_color = self.get_color(self.selected_label)
         self._color_mode = color_mode
         self.events.colormap()  # Will update the LabelVispyColormap shader
@@ -529,45 +532,12 @@ class Labels(_ImageBase):
         self.refresh()
 
     @property
-    def num_colors(self):
-        """int: Number of unique colors to use in colormap."""
-        warnings.warn(
-            trans._(
-                'Labels.num_colors is deprecated since 0.4.19 and will be '
-                'removed in 0.5.0, please use len(Labels.colormap) '
-                'instead.',
-                deferred=True,
-            ),
-            FutureWarning,
-            stacklevel=2,
-        )
-        return len(self.colormap)
-
-    @num_colors.setter
-    def num_colors(self, num_colors):
-        warnings.warn(
-            trans._(
-                'Setting Labels.num_colors is deprecated since 0.4.19 and '
-                'will be removed in 0.5.0, please set Labels.colormap '
-                'instead.',
-                deferred=True,
-            ),
-            FutureWarning,
-            stacklevel=2,
-        )
-        self.colormap = label_colormap(
-            num_colors - 1,
-            seed=self._random_colormap.seed,
-            background_value=self.colormap.background_value,
-        )
-
-    @property
-    def data(self):
+    def data(self) -> Union[LayerDataProtocol, MultiScaleData]:
         """array: Image data."""
         return self._data
 
     @data.setter
-    def data(self, data):
+    def data(self, data: Union[LayerDataProtocol, MultiScaleData]):
         data = self._ensure_int_labels(data)
         self._data = data
         self._ndim = len(self._data.shape)
@@ -580,7 +550,7 @@ class Labels(_ImageBase):
         """Dataframe-like features table.
 
         It is an implementation detail that this is a `pandas.DataFrame`. In the future,
-        we will target the currently-in-development Data API dataframe protocol [1].
+        we will target the currently-in-development Data API dataframe protocol [1]_.
         This will enable us to use alternate libraries such as xarray or cuDF for
         additional features without breaking existing usage of this.
 
@@ -589,14 +559,14 @@ class Labels(_ImageBase):
 
         References
         ----------
-        .. [1]: https://data-apis.org/dataframe-protocol/latest/API.html
+        .. [1] https://data-apis.org/dataframe-protocol/latest/API.html
         """
         return self._feature_table.values
 
     @features.setter
     def features(
         self,
-        features: Union[Dict[str, np.ndarray], pd.DataFrame],
+        features: Union[dict[str, np.ndarray], pd.DataFrame],
     ) -> None:
         self._feature_table.set_values(features)
         self._label_index = self._make_label_index()
@@ -604,15 +574,15 @@ class Labels(_ImageBase):
         self.events.features()
 
     @property
-    def properties(self) -> Dict[str, np.ndarray]:
+    def properties(self) -> dict[str, np.ndarray]:
         """dict {str: array (N,)}, DataFrame: Properties for each label."""
         return self._feature_table.properties()
 
     @properties.setter
-    def properties(self, properties: Dict[str, Array]):
+    def properties(self, properties: dict[str, Array]):
         self.features = properties
 
-    def _make_label_index(self) -> Dict[int, int]:
+    def _make_label_index(self) -> dict[int, int]:
         features = self._feature_table.values
         label_index = {}
         if 'index' in features:
@@ -621,35 +591,6 @@ class Labels(_ImageBase):
             label_index = {i: i for i in range(features.shape[0])}
         return label_index
 
-    @property
-    def color(self) -> dict:
-        """dict: custom color dict for label coloring"""
-        warnings.warn(
-            "Labels.color is deprecated since 0.4.19 and will be removed in "
-            "0.5.0, please use Labels.colormap.color_dict instead. Note: this"
-            "will only work when the colormap is a DirectLabelsColormap.",
-            FutureWarning,
-            stacklevel=2,
-        )
-        return {**self._direct_colormap.color_dict}
-
-    @color.setter
-    def color(self, color: Dict[Optional[int], Union[str, np.ndarray]]):
-        warnings.warn(
-            "Labels.color is deprecated since 0.4.19 and will be removed in "
-            "0.5.0, please set Labels.colormap directly with an instance "
-            "of napari.utils.colormaps.DirectLabelColormap instead.",
-            FutureWarning,
-            stacklevel=2,
-        )
-        color = dict(color) if color else {}
-
-        color[self.colormap.background_value] = color.get(
-            self.colormap.background_value, 'transparent'
-        )
-        color[None] = color.get(None, 'black')
-        self.colormap = DirectLabelColormap(color_dict=color)
-
     def _is_default_colors(self, color):
         """Returns True if color contains only default colors, otherwise False.
 
@@ -690,7 +631,7 @@ class Labels(_ImageBase):
             if np.issubdtype(normalize_dtype(data_level.dtype), np.floating):
                 raise TypeError(
                     trans._(
-                        "Only integer types are supported for Labels layers, but data contains {data_level_type}.",
+                        'Only integer types are supported for Labels layers, but data contains {data_level_type}.',
                         data_level_type=data_level.dtype,
                     )
                 )
@@ -703,12 +644,12 @@ class Labels(_ImageBase):
             data = data[0]
         return data
 
-    def _get_state(self):
+    def _get_state(self) -> dict[str, Any]:
         """Get dictionary of layer state.
 
         Returns
         -------
-        state : dict
+        state : dict of str to Any
             Dictionary of layer state.
         """
         state = self._get_base_state()
@@ -739,6 +680,7 @@ class Labels(_ImageBase):
         if selected_label == self.selected_label:
             return
 
+        self._prev_selected_label = self.selected_label
         self.colormap.selection = selected_label
         self._selected_label = selected_label
         self._selected_color = self.get_color(selected_label)
@@ -755,55 +697,6 @@ class Labels(_ImageBase):
         else:
             self.selected_label = self._prev_selected_label
 
-    @property
-    def color_mode(self):
-        """Color mode to change how color is represented.
-
-        AUTO (default) allows color to be set via a hash function with a seed.
-
-        DIRECT allows color of each label to be set directly by a color dict.
-        """
-        warnings.warn(
-            trans._(
-                'Labels.color_mode is deprecated since 0.4.19 and will be '
-                'removed in 0.5.0. Please check type(Labels.colormap) '
-                'instead. napari.utils.colormaps.CyclicLabelColormap '
-                'corresponds to AUTO color mode, and '
-                'napari.utils.colormaps.DirectLabelColormap'
-                ' corresponds to DIRECT color mode.',
-                deferred=True,
-            ),
-            FutureWarning,
-            stacklevel=2,
-        )
-        return str(self._color_mode)
-
-    @color_mode.setter
-    def color_mode(self, color_mode: Union[str, LabelColorMode]):
-        warnings.warn(
-            trans._(
-                'Labels.color_mode is deprecated since 0.4.19 and will be '
-                'removed in 0.5.0. Please set Labels.colormap instead, to an'
-                'instance of napari.utils.colormaps.CyclicLabelColormap for '
-                '"auto" mode, or napari.utils.colormaps.DirectLabelColormap '
-                'for "direct" mode.',
-                deferred=True,
-            ),
-            FutureWarning,
-            stacklevel=2,
-        )
-        color_mode = LabelColorMode(color_mode)
-        self._color_mode = color_mode
-        if color_mode == LabelColorMode.AUTO:
-            self._colormap = self._random_colormap
-        else:
-            self._colormap = self._direct_colormap
-        self._selected_color = self.get_color(self.selected_label)
-        self.events.color_mode()
-        self.events.colormap()  # If remove this emitting, connect shader update to color_mode
-        self.events.selected_label()
-        self.refresh()
-
     @property
     def show_selected_label(self):
         """Whether to filter displayed labels to only the selected label or not"""
@@ -817,7 +710,8 @@ class Labels(_ImageBase):
         self.events.show_selected_label(show_selected_label=show_selected)
         self.refresh()
 
-    @Layer.mode.getter
+    # Only overriding to change the docstring
+    @property
     def mode(self):
         """MODE: Interactive mode. The normal, default mode is PAN_ZOOM, which
         allows for normal interactivity with the canvas.
@@ -842,13 +736,19 @@ class Labels(_ImageBase):
         In ERASE mode the cursor functions similarly to PAINT mode, but to
         paint with background label, which effectively removes the label.
         """
-        return str(self._mode)
+        return Layer.mode.fget(self)
+
+    # Only overriding to change the docstring of the setter above
+    @mode.setter
+    def mode(self, mode):
+        Layer.mode.fset(self, mode)
 
     def _mode_setter_helper(self, mode):
         mode = super()._mode_setter_helper(mode)
         if mode == self._mode:
             return mode
 
+        self._overlays['polygon'].enabled = mode == Mode.POLYGON
         if mode in {Mode.PAINT, Mode.ERASE}:
             self.cursor_size = self._calculate_cursor_size()
 
@@ -887,6 +787,11 @@ class Labels(_ImageBase):
         """
         return vispy_texture_dtype(data)
 
+    def _update_slice_response(self, response: _ImageSliceResponse) -> None:
+        """Override to convert raw slice data to displayed label colors."""
+        response = response.to_displayed(self._raw_to_displayed)
+        super()._update_slice_response(response)
+
     def _partial_labels_refresh(self):
         """Prepares and displays only an updated part of the labels."""
 
@@ -921,7 +826,7 @@ class Labels(_ImageBase):
         self._updated_slice = None
 
     def _calculate_contour(
-        self, labels: np.ndarray, data_slice: Tuple[slice, ...]
+        self, labels: np.ndarray, data_slice: tuple[slice, ...]
     ) -> Optional[np.ndarray]:
         """Calculate the contour of a given label array within the specified data slice.
 
@@ -944,7 +849,7 @@ class Labels(_ImageBase):
         if labels.ndim > 2:
             warnings.warn(
                 trans._(
-                    "Contours are not displayed during 3D rendering",
+                    'Contours are not displayed during 3D rendering',
                     deferred=True,
                 )
             )
@@ -965,7 +870,7 @@ class Labels(_ImageBase):
         return sliced_labels[delta_slice]
 
     def _raw_to_displayed(
-        self, raw, data_slice: Optional[Tuple[slice, ...]] = None
+        self, raw, data_slice: Optional[tuple[slice, ...]] = None
     ) -> np.ndarray:
         """Determine displayed image from a saved raw image and a saved seed.
 
@@ -981,6 +886,7 @@ class Labels(_ImageBase):
             Slice that specifies the portion of the input image that
             should be computed and displayed.
             If None, the whole input image will be processed.
+
         Returns
         -------
         mapped_labels : array
@@ -1049,9 +955,9 @@ class Labels(_ImageBase):
 
     def _get_value_ray(
         self,
-        start_point: np.ndarray,
-        end_point: np.ndarray,
-        dims_displayed: List[int],
+        start_point: Optional[np.ndarray],
+        end_point: Optional[np.ndarray],
+        dims_displayed: list[int],
     ) -> Optional[int]:
         """Get the first non-background value encountered along a ray.
 
@@ -1106,7 +1012,8 @@ class Labels(_ImageBase):
             bounding_box[:, 1] += 1
 
             clamped = clamp_point_to_bounding_box(
-                sample_points, bounding_box
+                sample_points,
+                bounding_box,
             ).astype(int)
             values = im_slice[tuple(clamped.T)]
             nonzero_indices = np.flatnonzero(values)
@@ -1118,9 +1025,9 @@ class Labels(_ImageBase):
 
     def _get_value_3d(
         self,
-        start_point: np.ndarray,
-        end_point: np.ndarray,
-        dims_displayed: List[int],
+        start_point: Optional[np.ndarray],
+        end_point: Optional[np.ndarray],
+        dims_displayed: list[int],
     ) -> Optional[int]:
         """Get the first non-background value encountered along a ray.
 
@@ -1371,13 +1278,7 @@ class Labels(_ImageBase):
             Whether to refresh view slice or not. Set to False to batch paint
             calls.
         """
-        shape = self.data.shape
-        dims_to_paint = sorted(
-            self._slice_input.order[-self.n_edit_dimensions :]
-        )
-        dims_not_painted = sorted(
-            self._slice_input.order[: -self.n_edit_dimensions]
-        )
+        shape, dims_to_paint = self._get_shape_and_dims_to_paint()
         paint_scale = np.array(
             [self.scale[i] for i in dims_to_paint], dtype=float
         )
@@ -1385,7 +1286,6 @@ class Labels(_ImageBase):
         slice_coord = [int(np.round(c)) for c in coord]
         if self.n_edit_dimensions < self.ndim:
             coord_paint = [coord[i] for i in dims_to_paint]
-            shape = [shape[i] for i in dims_to_paint]
         else:
             coord_paint = coord
 
@@ -1398,6 +1298,71 @@ class Labels(_ImageBase):
             int
         )
 
+        self._paint_indices(
+            mask_indices, new_label, shape, dims_to_paint, slice_coord, refresh
+        )
+
+    def paint_polygon(self, points, new_label):
+        """Paint a polygon over existing labels with a new label.
+
+        Parameters
+        ----------
+        points : list of coordinates
+            List of coordinates of the vertices of a polygon.
+        new_label : int
+            Value of the new label to be filled in.
+        """
+        shape, dims_to_paint = self._get_shape_and_dims_to_paint()
+
+        if len(dims_to_paint) != 2:
+            raise NotImplementedError(
+                'Polygon painting is implemented only in 2D.'
+            )
+
+        points = np.array(points, dtype=int)
+        slice_coord = points[0].tolist()
+        points2d = points[:, dims_to_paint]
+
+        polygon_mask = polygon2mask(shape, points2d)
+        mask_indices = np.argwhere(polygon_mask)
+        self._paint_indices(
+            mask_indices,
+            new_label,
+            shape,
+            dims_to_paint,
+            slice_coord,
+            refresh=True,
+        )
+
+    def _paint_indices(
+        self,
+        mask_indices,
+        new_label,
+        shape,
+        dims_to_paint,
+        slice_coord=None,
+        refresh=True,
+    ):
+        """Paint over existing labels with a new label, using the selected
+        mask indices, either only on the visible slice or in all n dimensions.
+
+        Parameters
+        ----------
+        mask_indices : numpy array of integer coordinates
+            Mask to paint represented by an array of its coordinates.
+        new_label : int
+            Value of the new label to be filled in.
+        shape : list
+            The label data shape upon which painting is performed.
+        dims_to_paint : list
+            List of dimensions of the label data that are used for painting.
+        refresh : bool
+            Whether to refresh view slice or not. Set to False to batch paint
+            calls.
+        """
+        dims_not_painted = sorted(
+            self._slice_input.order[: -self.n_edit_dimensions]
+        )
         # discard candidate coordinates that are out of bounds
         mask_indices = indices_in_shape(mask_indices, shape)
 
@@ -1430,7 +1395,7 @@ class Labels(_ImageBase):
 
         self.data_setitem(slice_coord, new_label, refresh)
 
-    def _get_shape_and_dims_to_paint(self) -> Tuple[list, list]:
+    def _get_shape_and_dims_to_paint(self) -> tuple[list, list]:
         dims_to_paint = sorted(self._get_dims_to_paint())
         shape = list(self.data.shape)
 
@@ -1442,14 +1407,15 @@ class Labels(_ImageBase):
     def _get_dims_to_paint(self) -> list:
         return list(self._slice_input.order[-self.n_edit_dimensions :])
 
-    def _get_pt_not_disp(self) -> Dict[int, int]:
+    def _get_pt_not_disp(self) -> dict[int, int]:
         """
         Get indices of current visible slice.
         """
-        point = np.round(self.world_to_data(self._slice_input.point)).astype(
-            int
-        )
-        return {dim: point[dim] for dim in self._slice_input.not_displayed}
+        slice_input = self._slice.slice_input
+        point = np.round(
+            self.world_to_data(slice_input.world_slice.point)
+        ).astype(int)
+        return {dim: point[dim] for dim in slice_input.not_displayed}
 
     def data_setitem(self, indices, value, refresh=True):
         """Set `indices` in `data` to `value`, while writing to edit history.
@@ -1458,7 +1424,7 @@ class Labels(_ImageBase):
         ----------
         indices : tuple of arrays of int
             Indices in data to overwrite. Must be a tuple of arrays of length
-            equal to the number of data dimensions. (Fancy indexing in [1]_).
+            equal to the number of data dimensions. (Fancy indexing in [2]_).
         value : int or array of int
             New label value(s). If more than one value, must match or
             broadcast with the given indices.
@@ -1467,7 +1433,7 @@ class Labels(_ImageBase):
 
         References
         ----------
-        ..[1] https://numpy.org/doc/stable/user/basics.indexing.html
+        .. [2] https://numpy.org/doc/stable/user/basics.indexing.html
         """
         changed_indices = self.data[indices] != value
         indices = tuple(x[changed_indices] for x in indices)
@@ -1497,7 +1463,7 @@ class Labels(_ImageBase):
 
         pt_not_disp = self._get_pt_not_disp()
         displayed_indices = index_in_slice(
-            indices, pt_not_disp, self._slice_input.order
+            indices, pt_not_disp, self._slice.slice_input.order
         )
         if isinstance(value, np.ndarray):
             visible_values = value[elements_in_slice(indices, pt_not_disp)]
@@ -1532,11 +1498,11 @@ class Labels(_ImageBase):
             # the original slice because of the morphological dilation
             # (1 pixel because get_countours always applies 1 pixel dilation)
             updated_slice = expand_slice(updated_slice, self.data.shape, 1)
-        elif self.loaded:
+        else:
             # update data view
-            self._slice.image.view[
-                displayed_indices
-            ] = self.colormap._data_to_texture(visible_values)
+            self._slice.image.view[displayed_indices] = (
+                self.colormap._data_to_texture(visible_values)
+            )
 
         if self._updated_slice is None:
             self._updated_slice = updated_slice
@@ -1553,10 +1519,10 @@ class Labels(_ImageBase):
 
     def get_status(
         self,
-        position: Optional[Tuple] = None,
+        position: Optional[npt.ArrayLike] = None,
         *,
-        view_direction: Optional[np.ndarray] = None,
-        dims_displayed: Optional[List[int]] = None,
+        view_direction: Optional[npt.ArrayLike] = None,
+        dims_displayed: Optional[list[int]] = None,
         world: bool = False,
     ) -> dict:
         """Status message information of the data at a coordinate position.
@@ -1591,19 +1557,21 @@ class Labels(_ImageBase):
             value = None
 
         source_info = self._get_source_info()
-        source_info['coordinates'] = generate_layer_coords_status(
-            position[-self.ndim :], value
-        )
+
+        pos = position
+        if pos is not None:
+            pos = np.asarray(pos)[-self.ndim :]
+        source_info['coordinates'] = generate_layer_coords_status(pos, value)
 
         # if this labels layer has properties
         properties = self._get_properties(
             position,
-            view_direction=view_direction,
+            view_direction=np.asarray(view_direction),
             dims_displayed=dims_displayed,
             world=world,
         )
         if properties:
-            source_info['coordinates'] += "; " + ", ".join(properties)
+            source_info['coordinates'] += '; ' + ', '.join(properties)
 
         return source_info
 
@@ -1612,7 +1580,7 @@ class Labels(_ImageBase):
         position,
         *,
         view_direction: Optional[np.ndarray] = None,
-        dims_displayed: Optional[List[int]] = None,
+        dims_displayed: Optional[list[int]] = None,
         world: bool = False,
     ):
         """
@@ -1637,7 +1605,7 @@ class Labels(_ImageBase):
         msg : string
             String containing a message that can be used as a tooltip.
         """
-        return "\n".join(
+        return '\n'.join(
             self._get_properties(
                 position,
                 view_direction=view_direction,
@@ -1651,7 +1619,7 @@ class Labels(_ImageBase):
         position,
         *,
         view_direction: Optional[np.ndarray] = None,
-        dims_displayed: Optional[List[int]] = None,
+        dims_displayed: Optional[list[int]] = None,
         world: bool = False,
     ) -> list:
         if len(self._label_index) == 0 or self.features.shape[1] == 0:
@@ -1682,13 +1650,6 @@ class Labels(_ImageBase):
         ]
 
 
-if config.async_octree:
-    from napari.layers.image.experimental.octree_image import _OctreeImageBase
-
-    class Labels(Labels, _OctreeImageBase):
-        pass
-
-
 def _coerce_indices_for_vectorization(array, indices: list) -> tuple:
     """Coerces indices so that they can be used for vectorized indexing in the given data array."""
     if _is_array_type(array, 'xarray.DataArray'):
