@@ -20,7 +20,7 @@ def test_rectangle():
     np.random.seed(0)
     data = 20 * np.random.random((4, 2))
     shape = Rectangle(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (4, 2)
     assert shape.slice_key.shape == (2, 0)
 
@@ -39,7 +39,7 @@ def test_nD_rectangle():
     data = 20 * np.random.random((4, 3))
     data[:, 0] = 0
     shape = Rectangle(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (4, 2)
     assert shape.slice_key.shape == (2, 1)
 
@@ -64,7 +64,7 @@ def test_polygon_data_triangle():
 
 
 def test_polygon_data_triangle_module():
-    pytest.importorskip("triangle")
+    pytest.importorskip('triangle')
     data = np.array(
         [
             [10.97627008, 14.30378733],
@@ -94,11 +94,11 @@ def test_polygon():
         ]
     )
     shape = Polygon(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (6, 2)
     assert shape.slice_key.shape == (2, 0)
     # should get few triangles
-    expected_face = (6, 2) if "triangle" in sys.modules else (8, 2)
+    expected_face = (6, 2) if 'triangle' in sys.modules else (8, 2)
     assert shape._edge_vertices.shape == (16, 2)
     assert shape._face_vertices.shape == expected_face
 
@@ -108,7 +108,7 @@ def test_polygon2():
     shape = Polygon(data, interpolation_order=3)
     # should get many triangles
 
-    expected_face = (249, 2) if "triangle" in sys.modules else (251, 2)
+    expected_face = (249, 2) if 'triangle' in sys.modules else (251, 2)
 
     assert shape._edge_vertices.shape == (500, 2)
     assert shape._face_vertices.shape == expected_face
@@ -130,7 +130,7 @@ def test_nD_polygon():
     data = 20 * np.random.random((6, 3))
     data[:, 0] = 0
     shape = Polygon(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (6, 2)
     assert shape.slice_key.shape == (2, 1)
 
@@ -144,7 +144,7 @@ def test_path():
     np.random.seed(0)
     data = 20 * np.random.random((6, 2))
     shape = Path(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (6, 2)
     assert shape.slice_key.shape == (2, 0)
 
@@ -155,7 +155,7 @@ def test_nD_path():
     np.random.seed(0)
     data = 20 * np.random.random((6, 3))
     shape = Path(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (6, 2)
     assert shape.slice_key.shape == (2, 1)
 
@@ -169,7 +169,7 @@ def test_line():
     np.random.seed(0)
     data = 20 * np.random.random((2, 2))
     shape = Line(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (2, 2)
     assert shape.slice_key.shape == (2, 0)
 
@@ -180,7 +180,7 @@ def test_nD_line():
     np.random.seed(0)
     data = 20 * np.random.random((2, 3))
     shape = Line(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (2, 2)
     assert shape.slice_key.shape == (2, 1)
 
@@ -194,7 +194,7 @@ def test_ellipse():
     np.random.seed(0)
     data = 20 * np.random.random((4, 2))
     shape = Ellipse(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (4, 2)
     assert shape.slice_key.shape == (2, 0)
 
@@ -213,7 +213,7 @@ def test_nD_ellipse():
     data = 20 * np.random.random((4, 3))
     data[:, 0] = 0
     shape = Ellipse(data)
-    assert np.all(shape.data == data)
+    np.testing.assert_array_equal(shape.data, data)
     assert shape.data_displayed.shape == (4, 2)
     assert shape.slice_key.shape == (2, 1)
 
