@@ -1,4 +1,4 @@
-from typing import Dict, List, Union
+from typing import TYPE_CHECKING, Optional, Union
 
 import numpy as np
 import pandas as pd
@@ -9,22 +9,18 @@ from napari.layers.utils.layer_utils import _FeatureTable
 from napari.utils.events.custom_types import Array
 from napari.utils.translations import trans
 
-
-def connex(vertices: np.ndarray) -> list:
-    """Connection array to build vertex edges for vispy LineVisual.
-
-    Notes
-    -----
-    See
-    http://api.vispy.org/en/latest/visuals.html#vispy.visuals.LineVisual
-
-    """
-    return [True] * (vertices.shape[0] - 1) + [False]
+if TYPE_CHECKING:
+    import numpy.typing as npt
 
 
 class TrackManager:
     """Manage track data and simplify interactions with the Tracks layer.
 
+    Parameters
+    ----------
+    data : array
+        See attribute doc below.
+
     Attributes
     ----------
     data : array (N, D+1)
@@ -66,33 +62,28 @@ class TrackManager:
         Track ID for each vertex in track_vertices.
     """
 
-    def __init__(self) -> None:
+    def __init__(self, data: np.ndarray) -> None:
         # store the raw data here
-        self._data = None
+        self.data = data
         self._feature_table = _FeatureTable()
-        self._order = None
 
-        # use a kdtree to help with fast lookup of the nearest track
-        self._kdtree = None
-
-        # NOTE(arl): _tracks and _connex store raw data for vispy
-        self._points = None
-        self._points_id = None
-        self._points_lookup = None
-        self._ordered_points_idx = None
+        self._data: npt.NDArray
+        self._order: list[int]
+        self._kdtree: cKDTree
+        self._points: npt.NDArray
+        self._points_id: npt.NDArray
+        self._points_lookup: dict[int, slice]
+        self._ordered_points_idx: npt.NDArray
 
         self._track_vertices = None
         self._track_connex = None
 
-        self._graph = None
+        self._graph: Optional[dict[int, list[int]]] = None
         self._graph_vertices = None
         self._graph_connex = None
 
-        # lookup table for vertex indices from track id
-        self._id2idxs = None
-
     @staticmethod
-    def _fast_points_lookup(sorted_time: np.ndarray) -> Dict[int, slice]:
+    def _fast_points_lookup(sorted_time: np.ndarray) -> dict[int, slice]:
         """Computes a fast lookup table from time to their respective points slicing."""
 
         # finds where t transitions to t + 1
@@ -128,8 +119,8 @@ class TrackManager:
         self._data = data[self._order]
 
         # build the indices for sorting points by time
-        self._ordered_points_idx = np.argsort(self.data[:, 1])
-        self._points = self.data[self._ordered_points_idx, 1:]
+        self._ordered_points_idx = np.argsort(self._data[:, 1])
+        self._points = self._data[self._ordered_points_idx, 1:]
 
         # build a tree of the track data to allow fast lookup of nearest track
         self._kdtree = cKDTree(self._points)
@@ -172,7 +163,7 @@ class TrackManager:
     @features.setter
     def features(
         self,
-        features: Union[Dict[str, np.ndarray], pd.DataFrame],
+        features: Union[dict[str, np.ndarray], pd.DataFrame],
     ) -> None:
         self._feature_table.set_values(features, num_data=len(self.data))
         self._feature_table.reorder(self._order)
@@ -180,24 +171,24 @@ class TrackManager:
             self._feature_table.values['track_id'] = self.track_ids
 
     @property
-    def properties(self) -> Dict[str, np.ndarray]:
+    def properties(self) -> dict[str, np.ndarray]:
         """dict {str: np.ndarray (N,)}: Properties for each track."""
         return self._feature_table.properties()
 
     @properties.setter
-    def properties(self, properties: Dict[str, Array]):
+    def properties(self, properties: dict[str, Array]):
         """set track properties"""
         self.features = properties
 
     @property
-    def graph(self) -> Dict[int, Union[int, List[int]]]:
+    def graph(self) -> Optional[dict[int, list[int]]]:
         """dict {int: list}: Graph representing associations between tracks."""
         return self._graph
 
     @graph.setter
-    def graph(self, graph: Dict[int, Union[int, List[int]]]):
+    def graph(self, graph: dict[int, Union[int, list[int]]]):
         """set the track graph"""
-        self._graph = self._validate_track_graph(graph)
+        self._graph = self._normalize_track_graph(graph)
 
     @property
     def track_ids(self):
@@ -235,7 +226,7 @@ class TrackManager:
 
         # check that all IDs are integers
         ids = data[:, 0]
-        if not np.all(np.floor(ids) == ids):
+        if not np.array_equal(np.floor(ids), ids):
             raise ValueError(
                 trans._('track id must be an integer', deferred=True)
             )
@@ -249,21 +240,24 @@ class TrackManager:
 
         return data
 
-    def _validate_track_graph(
-        self, graph: Dict[int, Union[int, List[int]]]
-    ) -> Dict[int, List[int]]:
+    def _normalize_track_graph(
+        self, graph: dict[int, Union[int, list[int]]]
+    ) -> dict[int, list[int]]:
         """validate the track graph"""
+        new_graph: dict[int, list[int]] = {}
 
         # check that graph nodes are of correct format
         for node_idx, parents_idx in graph.items():
             # make sure parents are always a list
-            if type(parents_idx) != list:  # noqa: E721
-                graph[node_idx] = [parents_idx]
+            if isinstance(parents_idx, list):
+                new_graph[node_idx] = parents_idx
+            else:
+                new_graph[node_idx] = [parents_idx]
 
         unique_track_ids = set(self.unique_track_ids)
 
         # check that graph nodes exist in the track id lookup
-        for node_idx, parents_idx in graph.items():
+        for node_idx, parents_idx in new_graph.items():
             nodes = [node_idx, *parents_idx]
             for node in nodes:
                 if node not in unique_track_ids:
@@ -275,30 +269,29 @@ class TrackManager:
                         )
                     )
 
-        return graph
+        return new_graph
 
     def build_tracks(self):
         """build the tracks"""
 
-        points_id = []
-        track_vertices = []
-        track_connex = []
+        # Track ids associated to all vertices, sorted by time
+        points_id = self.data[:, 0][self._ordered_points_idx]
+        # Coordinates of all vertices
+        track_vertices = self.data[:, 1:]
 
-        # NOTE(arl): this takes some time when the number of tracks is large
-        for idx in self.unique_track_ids:
-            indices = self._vertex_indices_from_id(idx)
+        # Indices in the data array just before the track id changes
+        indices_new_id = np.where(np.diff(self.data[:, 0]))[0]
 
-            # grab the correct vertices and sort by time
-            vertices = self.data[indices, 1:]
+        # Define track_connex as an array full of 'True', then set to 'False'
+        # at the indices just before the track id changes
+        track_connex = np.ones(self.data.shape[0], dtype=bool)
+        track_connex[indices_new_id] = False
+        # Add 'False' for the last entry too (end of the last track)
+        track_connex[-1] = False
 
-            # coordinates of the text identifiers, vertices and connections
-            points_id += [idx] * vertices.shape[0]
-            track_vertices.append(vertices)
-            track_connex.append(connex(vertices))
-
-        self._points_id = np.array(points_id)[self._ordered_points_idx]
-        self._track_vertices = np.concatenate(track_vertices, axis=0)
-        self._track_connex = np.concatenate(track_connex, axis=0)
+        self._points_id = points_id
+        self._track_vertices = track_vertices
+        self._track_connex = track_connex
 
     def build_graph(self):
         """build the track graph"""
@@ -365,22 +358,24 @@ class TrackManager:
         return self.data.shape[1] - 1
 
     @property
-    def max_time(self) -> int:
+    def max_time(self) -> Optional[int]:
         """Determine the maximum timestamp of the dataset"""
-        return int(np.max(self.track_times))
+        if self.track_times is not None:
+            return int(np.max(self.track_times))
+        return None
 
     @property
-    def track_vertices(self) -> np.ndarray:
+    def track_vertices(self) -> Optional[np.ndarray]:
         """return the track vertices"""
         return self._track_vertices
 
     @property
-    def track_connex(self) -> np.ndarray:
+    def track_connex(self) -> Optional[np.ndarray]:
         """vertex connections for drawing track lines"""
         return self._track_connex
 
     @property
-    def graph_vertices(self) -> np.ndarray:
+    def graph_vertices(self) -> Optional[np.ndarray]:
         """return the graph vertices"""
         return self._graph_vertices
 
@@ -390,24 +385,32 @@ class TrackManager:
         return self._graph_connex
 
     @property
-    def track_times(self) -> np.ndarray:
+    def track_times(self) -> Optional[np.ndarray]:
         """time points associated with each track vertex"""
-        return self.track_vertices[:, 0]
+        if self.track_vertices is not None:
+            return self.track_vertices[:, 0]
+        return None
 
     @property
-    def graph_times(self) -> np.ndarray:
+    def graph_times(self) -> Optional[np.ndarray]:
         """time points associated with each graph vertex"""
         if self.graph_vertices is not None:
             return self.graph_vertices[:, 0]
         return None
 
-    def track_labels(self, current_time: int) -> tuple:
+    def track_labels(
+        self, current_time: int
+    ) -> Union[tuple[None, None], tuple[list[str], np.ndarray]]:
         """return track labels at the current time"""
+        if self._points_id is None:
+            return None, None
         # this is the slice into the time ordered points array
         if current_time not in self._points_lookup:
-            return [], []
+            lbl = []
+            pos = np.array([])
+        else:
+            lookup = self._points_lookup[current_time]
+            pos = self._points[lookup, ...]
+            lbl = [f'ID:{i}' for i in self._points_id[lookup]]
 
-        lookup = self._points_lookup[current_time]
-        pos = self._points[lookup, ...]
-        lbl = [f'ID:{i}' for i in self._points_id[lookup]]
         return lbl, pos
