@@ -7,7 +7,7 @@ from napari.layers.utils.string_encoding import (
     DirectStringEncoding,
     FormatStringEncoding,
     ManualStringEncoding,
-    validate_string_encoding,
+    StringEncoding,
 )
 
 
@@ -15,7 +15,7 @@ def make_features_with_no_columns(*, num_rows) -> pd.DataFrame:
     return pd.DataFrame({}, index=range(num_rows))
 
 
-@pytest.fixture
+@pytest.fixture()
 def features() -> pd.DataFrame:
     return pd.DataFrame(
         {
@@ -25,7 +25,7 @@ def features() -> pd.DataFrame:
     )
 
 
-@pytest.fixture
+@pytest.fixture()
 def numeric_features() -> pd.DataFrame:
     return pd.DataFrame(
         {
@@ -117,7 +117,7 @@ def test_format(features):
 
 def test_format_with_bad_string(features):
     encoding = FormatStringEncoding(format='{class}: {confidence:.2f')
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match='unmatched'):
         encoding(features)
 
 
@@ -137,16 +137,29 @@ def test_validate_from_format_string():
     argument = '{class}: {score:.2f}'
     expected = FormatStringEncoding(format=argument)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
 
+def test_format_with_index(features):
+    encoding = FormatStringEncoding(format='{index}: {confidence:.2f}')
+    values = encoding(features)
+    np.testing.assert_array_equal(values, ['0: 0.50', '1: 1.00', '2: 0.25'])
+
+
+def test_format_with_index_column(features):
+    features['index'] = features['class']
+    encoding = FormatStringEncoding(format='{index}: {confidence:.2f}')
+    values = encoding(features)
+    np.testing.assert_array_equal(values, ['a: 0.50', 'b: 1.00', 'c: 0.25'])
+
+
 def test_validate_from_non_format_string():
     argument = 'abc'
     expected = DirectStringEncoding(feature=argument)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
@@ -155,7 +168,7 @@ def test_validate_from_sequence():
     argument = ['a', 'b', 'c']
     expected = ManualStringEncoding(array=argument)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
@@ -165,7 +178,7 @@ def test_validate_from_constant_dict():
     argument = {'constant': constant}
     expected = ConstantStringEncoding(constant=constant)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
@@ -176,7 +189,7 @@ def test_validate_from_manual_dict():
     argument = {'array': array, 'default': default}
     expected = ManualStringEncoding(array=array, default=default)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
@@ -186,16 +199,16 @@ def test_validate_from_direct_dict():
     argument = {'feature': feature}
     expected = DirectStringEncoding(feature=feature)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
 
 
 def test_validate_from_format_dict():
-    format = '{class}: {score:.2f}'
-    argument = {'format': format}
-    expected = FormatStringEncoding(format=format)
+    format_str = '{class}: {score:.2f}'
+    argument = {'format': format_str}
+    expected = FormatStringEncoding(format=format_str)
 
-    actual = validate_string_encoding(argument)
+    actual = StringEncoding.validate(argument)
 
     assert actual == expected
