@@ -13,9 +13,9 @@ from typing import Any, Union
 import numpy as np
 import pandas as pd
 import pytest
-from pydantic import Field
 
-from napari.layers.utils._style_encoding import (
+from napari._pydantic_compat import Field
+from napari.layers.utils.style_encoding import (
     _ConstantStyleEncoding,
     _DerivedStyleEncoding,
     _ManualStyleEncoding,
@@ -23,7 +23,7 @@ from napari.layers.utils._style_encoding import (
 from napari.utils.events.custom_types import Array
 
 
-@pytest.fixture
+@pytest.fixture()
 def features() -> pd.DataFrame:
     return pd.DataFrame(
         {
@@ -41,9 +41,9 @@ class ScalarConstantEncoding(_ConstantStyleEncoding[Scalar, ScalarArray]):
     constant: Scalar
 
 
-def test_scalar_constant_encoding_update(features):
+def test_scalar_constant_encoding_apply(features):
     encoding = ScalarConstantEncoding(constant=0)
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, 0)
 
 
@@ -70,21 +70,21 @@ class ScalarManualEncoding(_ManualStyleEncoding[Scalar, ScalarArray]):
     default: Scalar = np.array(-1)
 
 
-def test_scalar_manual_encoding_update_with_shorter(features):
+def test_scalar_manual_encoding_apply_with_shorter(features):
     encoding = ScalarManualEncoding(array=[1, 2, 3, 4])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [1, 2, 3])
 
 
-def test_scalar_manual_encoding_update_with_equal_length(features):
+def test_scalar_manual_encoding_apply_with_equal_length(features):
     encoding = ScalarManualEncoding(array=[1, 2, 3])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [1, 2, 3])
 
 
-def test_scalar_manual_encoding_update_with_longer(features):
+def test_scalar_manual_encoding_apply_with_longer(features):
     encoding = ScalarManualEncoding(array=[1, 2], default=-1)
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [1, 2, -1])
 
 
@@ -114,20 +114,20 @@ class ScalarDirectEncoding(_DerivedStyleEncoding[Scalar, ScalarArray]):
         return ScalarArray.validate_type(features[self.feature])
 
 
-def test_scalar_derived_encoding_update(features):
+def test_scalar_derived_encoding_apply(features):
     encoding = ScalarDirectEncoding(feature='scalar')
 
-    encoding._update(features)
+    encoding._apply(features)
 
     expected_values = features['scalar']
     np.testing.assert_array_equal(encoding._values, expected_values)
 
 
-def test_scalar_derived_encoding_update_with_failure(features):
+def test_scalar_derived_encoding_apply_with_failure(features):
     encoding = ScalarDirectEncoding(feature='not_a_column', fallback=-1)
 
     with pytest.warns(RuntimeWarning):
-        encoding._update(features)
+        encoding._apply(features)
 
     np.testing.assert_array_equal(encoding._values, [-1] * len(features))
 
@@ -167,9 +167,9 @@ class VectorConstantEncoding(_ConstantStyleEncoding[Vector, VectorArray]):
     constant: Vector
 
 
-def test_vector_constant_encoding_update(features):
+def test_vector_constant_encoding_apply(features):
     encoding = VectorConstantEncoding(constant=[0, 0])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [0, 0])
 
 
@@ -196,21 +196,21 @@ class VectorManualEncoding(_ManualStyleEncoding[Vector, VectorArray]):
     default: Vector = Field(default_factory=lambda: np.array([-1, -1]))
 
 
-def test_vector_manual_encoding_update_with_shorter(features):
+def test_vector_manual_encoding_apply_with_shorter(features):
     encoding = VectorManualEncoding(array=[[1, 1], [2, 2], [3, 3], [4, 4]])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [[1, 1], [2, 2], [3, 3]])
 
 
-def test_vector_manual_encoding_update_with_equal_length(features):
+def test_vector_manual_encoding_apply_with_equal_length(features):
     encoding = VectorManualEncoding(array=[[1, 1], [2, 2], [3, 3]])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [[1, 1], [2, 2], [3, 3]])
 
 
-def test_vector_manual_encoding_update_with_longer(features):
+def test_vector_manual_encoding_apply_with_longer(features):
     encoding = VectorManualEncoding(array=[[1, 1], [2, 2]], default=[-1, -1])
-    encoding._update(features)
+    encoding._apply(features)
     np.testing.assert_array_equal(encoding._values, [[1, 1], [2, 2], [-1, -1]])
 
 
@@ -242,20 +242,20 @@ class VectorDirectEncoding(_DerivedStyleEncoding[Vector, VectorArray]):
         return VectorArray.validate_type(list(features[self.feature]))
 
 
-def test_vector_derived_encoding_update(features):
+def test_vector_derived_encoding_apply(features):
     encoding = VectorDirectEncoding(feature='vector')
 
-    encoding._update(features)
+    encoding._apply(features)
 
     expected_values = list(features['vector'])
     np.testing.assert_array_equal(encoding._values, expected_values)
 
 
-def test_vector_derived_encoding_update_with_failure(features):
+def test_vector_derived_encoding_apply_with_failure(features):
     encoding = VectorDirectEncoding(feature='not_a_column', fallback=[-1, -1])
 
     with pytest.warns(RuntimeWarning):
-        encoding._update(features)
+        encoding._apply(features)
 
     np.testing.assert_array_equal(encoding._values, [[-1, -1]] * len(features))
 
