@@ -1,5 +1,6 @@
 import os
-from importlib.metadata import distribution
+import sys
+from importlib.metadata import PackageNotFoundError, distribution
 from unittest.mock import patch
 
 import pytest
@@ -7,38 +8,37 @@ import pytest
 from napari.settings import NapariSettings, _migrations
 
 
-@pytest.fixture
-def _test_migrator(monkeypatch):
+@pytest.fixture()
+def test_migrator(monkeypatch):
     # this fixture makes sure we're not using _migrations.MIGRATORS for tests
     # but rather only using migrators that get declared IN the test
     _TEST_MIGRATORS = []
     with monkeypatch.context() as m:
-        m.setattr(_migrations, "_MIGRATORS", _TEST_MIGRATORS)
+        m.setattr(_migrations, '_MIGRATORS', _TEST_MIGRATORS)
         yield _migrations.migrator
 
 
-def test_no_migrations_available(_test_migrator):
+def test_no_migrations_available(test_migrator):
     # no migrators exist... nothing should happen
     settings = NapariSettings(schema_version='0.1.0')
     assert settings.schema_version == '0.1.0'
 
 
-def test_backwards_migrator(_test_migrator):
+def test_backwards_migrator(test_migrator):
     # we shouldn't be able to downgrade the schema version
     # if that is needed later, we can create a new decorator,
     # or change this test
     with pytest.raises(AssertionError):
 
-        @_test_migrator('0.2.0', '0.1.0')
-        def _(model):
-            ...
+        @test_migrator('0.2.0', '0.1.0')
+        def _(model): ...
 
 
-def test_migration_works(_test_migrator):
+def test_migration_works(test_migrator):
     # test that a basic migrator works to change the version
     # and mutate the model
 
-    @_test_migrator('0.1.0', '0.2.0')
+    @test_migrator('0.1.0', '0.2.0')
     def _(model: NapariSettings):
         model.appearance.theme = 'light'
 
@@ -47,10 +47,9 @@ def test_migration_works(_test_migrator):
     assert settings.appearance.theme == 'light'
 
 
-def test_migration_saves(_test_migrator):
-    @_test_migrator('0.1.0', '0.2.0')
-    def _(model: NapariSettings):
-        ...
+def test_migration_saves(test_migrator):
+    @test_migrator('0.1.0', '0.2.0')
+    def _(model: NapariSettings): ...
 
     with patch.object(NapariSettings, 'save') as mock:
         mock.assert_not_called()
@@ -59,10 +58,10 @@ def test_migration_saves(_test_migrator):
         mock.assert_called()
 
 
-def test_failed_migration_leaves_version(_test_migrator):
+def test_failed_migration_leaves_version(test_migrator):
     # if an error occurs IN the migrator, the version should stay
     # where it was before the migration, and any changes reverted.
-    @_test_migrator('0.1.0', '0.2.0')
+    @test_migrator('0.1.0', '0.2.0')
     def _(model: NapariSettings):
         model.appearance.theme = 'light'
         assert model.appearance.theme == 'light'
@@ -87,7 +86,7 @@ def test_030_to_040_migration():
     try:
         d = distribution('napari-svg')
         assert 'napari.manifest' in {ep.group for ep in d.entry_points}
-    except Exception:
+    except PackageNotFoundError:
         pytest.fail(
             'napari-svg not present as an npe2 plugin. '
             'This test needs updating'
@@ -115,3 +114,51 @@ def test_040_to_050_migration():
     )
     assert '.tif' not in settings.plugins.extension2reader
     assert '*.tif' in settings.plugins.extension2reader
+
+
+@pytest.mark.skipif(sys.platform != 'darwin', reason='tests migration on macs')
+def test_050_to_060_migration_mac():
+    """Check that Ctrl and Meta are swapped on macOS when migrating."""
+    settings050 = NapariSettings(
+        schema_version='0.5.0',
+        shortcuts={
+            'shortcuts': {
+                'napari:focus_axes_up': ['Alt-Up'],
+                'napari:roll_axes': ['Control-E'],
+                'napari:transpose_axes': ['Control-Meta-T'],
+                'napari:paste_shape': ['V', 'Meta-T'],
+            }
+        },
+    )
+    settings060 = NapariSettings(
+        schema_version='0.6.0',
+        shortcuts={
+            'shortcuts': {
+                'napari:focus_axes_up': ['Alt-Up'],
+                'napari:roll_axes': ['Meta-E'],
+                'napari:transpose_axes': ['Ctrl-Meta-T'],
+                'napari:paste_shape': ['V', 'Ctrl-T'],
+            }
+        },
+    )
+    assert settings050 == settings060
+
+
+@pytest.mark.skipif(
+    sys.platform == 'darwin', reason='migration should not be no-op on macs'
+)
+def test_050_to_060_migration_linux_win():
+    """Check that shortcuts are unchanged on non-macOS when migrating."""
+    shortcuts_dict = {
+        'napari:focus_axes_up': ['Alt-Up'],
+        'napari:roll_axes': ['Control-E'],
+        'napari:transpose_axes': ['Control-Meta-T'],
+        'napari:paste_shape': ['V', 'Meta-T'],
+    }
+    settings050 = NapariSettings(
+        schema_version='0.5.0', shortcuts={'shortcuts': shortcuts_dict}
+    )
+    settings060 = NapariSettings(
+        schema_version='0.6.0', shortcuts={'shortcuts': shortcuts_dict}
+    )
+    assert settings050 == settings060
