@@ -18,10 +18,14 @@ from napari.utils.colormaps.standardize_color import transform_color
 from napari.utils.colormaps.vendored import cm
 
 
-@pytest.mark.parametrize("name", list(AVAILABLE_COLORMAPS.keys()))
+@pytest.mark.parametrize('name', list(AVAILABLE_COLORMAPS.keys()))
 def test_colormap(name):
-    np.random.seed(0)
+    if name in {'label_colormap', 'custom'}:
+        pytest.skip(
+            'label_colormap  and custom are inadvertantly added to AVAILABLE_COLORMAPS but is not a normal colormap'
+        )
 
+    np.random.seed(0)
     cmap = AVAILABLE_COLORMAPS[name]
 
     # Test can map random 0-1 values
@@ -106,7 +110,7 @@ def test_can_accept_named_mpl_colormap():
     assert cmap.name == cmap_name
 
 
-@pytest.mark.filterwarnings("ignore::UserWarning")
+@pytest.mark.filterwarnings('ignore::UserWarning')
 def test_can_accept_vispy_colormaps_in_dict():
     """Test that we can accept vispy colormaps in a dictionary."""
     colors_a = np.array([[0, 0, 0, 1], [0, 1, 0, 1], [0, 0, 1, 1]])
@@ -119,7 +123,7 @@ def test_can_accept_vispy_colormaps_in_dict():
     assert cmap.name == 'a'
 
 
-@pytest.mark.filterwarnings("ignore::UserWarning")
+@pytest.mark.filterwarnings('ignore::UserWarning')
 def test_can_accept_napari_colormaps_in_dict():
     """Test that we can accept vispy colormaps in a dictionary"""
     colors_a = np.array([[0, 0, 0, 1], [0, 1, 0, 1], [0, 0, 1, 1]])
@@ -164,7 +168,7 @@ def test_mpl_colormap_exists():
 
 
 @pytest.mark.parametrize(
-    "name,display_name",
+    ('name', 'display_name'),
     [
         ('twilight_shifted', 'twilight shifted'),  # MPL
         ('light_blues', 'light blues'),  # Vispy
@@ -175,7 +179,7 @@ def test_colormap_error_suggestion(name, display_name):
     Test that vispy/mpl errors, when using `display_name`, suggest `name`.
     """
     with pytest.raises(
-        KeyError, match=rf"{display_name}.*you might want to use.*{name}"
+        KeyError, match=rf'{display_name}.*you might want to use.*{name}'
     ):
         vispy_or_mpl_colormap(display_name)
 
@@ -185,7 +189,7 @@ def test_colormap_error_from_inexistent_name():
     Test that vispy/mpl errors when using a wrong name.
     """
     name = 'foobar'
-    with pytest.raises(KeyError, match=rf"{name}.*Recognized colormaps are"):
+    with pytest.raises(KeyError, match=rf'{name}.*Recognized colormaps are'):
         vispy_or_mpl_colormap(name)
 
 
@@ -231,3 +235,42 @@ def test_ensure_colormap_with_multi_colors(colors):
     expected_colors = transform_color(colors)
     np.testing.assert_array_equal(colormap.colors, expected_colors)
     assert re.match(r'\[unnamed colormap \d+\]', colormap.name) is not None
+
+
+@pytest.mark.parametrize('color', ['#abc', '#abcd', '#abcdef', '#00ABCDEF'])
+def test_ensure_colormap_with_hex_color_string(color):
+    """
+    Test all the accepted hex color representations (single/double digit rgb with/without alpha)
+    """
+    cmap = ensure_colormap(color)
+    assert isinstance(cmap, Colormap)
+    assert cmap.name == color.lower()
+
+
+@pytest.mark.parametrize('color', ['#f0f', '#f0fF', '#ff00ff', '#ff00ffFF'])
+def test_ensure_colormap_with_recognized_hex_color_string(color):
+    """
+    Test that a hex color string for magenta is associated with the existing magenta colormap
+    """
+    cmap = ensure_colormap(color)
+    assert isinstance(cmap, Colormap)
+    assert cmap.name == 'magenta'
+
+
+def test_ensure_colormap_error_with_invalid_hex_color_string():
+    """
+    Test that ensure_colormap errors when using an invalid hex color string
+    """
+    color = '#ff'
+    with pytest.raises(KeyError, match=rf'{color}.*Recognized colormaps are'):
+        ensure_colormap(color)
+
+
+@pytest.mark.parametrize('mpl_name', ['chartreuse', 'chocolate', 'lavender'])
+def test_ensure_colormap_with_recognized_mpl_color_name(mpl_name):
+    """
+    Test that the colormap name is identical to the the mpl color name passed to ensure_colormap
+    """
+    cmap = ensure_colormap(mpl_name)
+    assert isinstance(cmap, Colormap)
+    assert cmap.name == mpl_name
