@@ -2,10 +2,11 @@
 # pygments - see here for examples https://help.farbox.com/pygments.html
 import logging
 import re
+import sys
 import warnings
 from ast import literal_eval
 from contextlib import suppress
-from typing import List, Union
+from typing import Any, Literal, Optional, Union, overload
 
 import npe2
 
@@ -23,7 +24,7 @@ from napari.utils.translations import trans
 try:
     from qtpy import QT_VERSION
 
-    major, minor, *_ = QT_VERSION.split('.')
+    major, minor, *_ = QT_VERSION.split('.')  # type: ignore[attr-defined]
     use_gradients = (int(major) >= 5) and (int(minor) >= 12)
     del major, minor, QT_VERSION
 except (ImportError, RuntimeError):
@@ -63,6 +64,8 @@ class Theme(EventedModel):
         Color used to indicate something is wrong or could stop functionality.
     current : Color
         Color used to highlight Qt widget.
+    font_size : str
+        Font size (in points, pt) used in the application.
     """
 
     id: str
@@ -80,62 +83,91 @@ class Theme(EventedModel):
     warning: Color
     error: Color
     current: Color
+    font_size: str = '12pt' if sys.platform == 'darwin' else '9pt'
 
-    @validator("syntax_style", pre=True)
-    def _ensure_syntax_style(value: str) -> str:
+    @validator('syntax_style', pre=True, allow_reuse=True)
+    def _ensure_syntax_style(cls, value: str) -> str:
         from pygments.styles import STYLE_MAP
 
         assert value in STYLE_MAP, trans._(
-            "Incorrect `syntax_style` value: {value} provided. Please use one of the following: {syntax_style}",
+            'Incorrect `syntax_style` value: {value} provided. Please use one of the following: {syntax_style}',
             deferred=True,
             syntax_style=f" {', '.join(STYLE_MAP)}",
             value=value,
         )
         return value
 
+    @validator('font_size', pre=True)
+    def _ensure_font_size(cls, value: str) -> str:
+        assert value.endswith('pt'), trans._(
+            'Font size must be in points (pt).', deferred=True
+        )
+        assert int(value[:-2]) > 0, trans._(
+            'Font size must be greater than 0.', deferred=True
+        )
+        return value
 
+    def to_rgb_dict(self) -> dict[str, Any]:
+        """
+        This differs from baseclass `dict()` by converting colors to rgb.
+        """
+        th = super().dict()
+        return {
+            k: v if not isinstance(v, Color) else v.as_rgb()
+            for (k, v) in th.items()
+        }
+
+
+increase_pattern = re.compile(r'{{\s?increase\((\w+),?\s?([-\d]+)?\)\s?}}')
+decrease_pattern = re.compile(r'{{\s?decrease\((\w+),?\s?([-\d]+)?\)\s?}}')
 gradient_pattern = re.compile(r'([vh])gradient\((.+)\)')
 darken_pattern = re.compile(r'{{\s?darken\((\w+),?\s?([-\d]+)?\)\s?}}')
 lighten_pattern = re.compile(r'{{\s?lighten\((\w+),?\s?([-\d]+)?\)\s?}}')
 opacity_pattern = re.compile(r'{{\s?opacity\((\w+),?\s?([-\d]+)?\)\s?}}')
 
 
-def darken(color: Union[str, Color], percentage=10):
-    if isinstance(color, str) and color.startswith('rgb('):
-        color = literal_eval(color.lstrip('rgb(').rstrip(')'))
-    else:
-        color = color.as_rgb_tuple()
+def decrease(font_size: str, pt: int) -> str:
+    """Decrease fontsize."""
+    return f'{int(font_size[:-2]) - int(pt)}pt'
+
+
+def increase(font_size: str, pt: int) -> str:
+    """Increase fontsize."""
+    return f'{int(font_size[:-2]) + int(pt)}pt'
+
+
+def _parse_color_as_rgb(color: Union[str, Color]) -> tuple[int, int, int]:
+    if isinstance(color, str):
+        if color.startswith('rgb('):
+            return literal_eval(color.lstrip('rgb(').rstrip(')'))
+        return Color(color).as_rgb_tuple()[:3]
+    return color.as_rgb_tuple()[:3]
+
+
+def darken(color: Union[str, Color], percentage: float = 10) -> str:
     ratio = 1 - float(percentage) / 100
-    red, green, blue = color
+    red, green, blue = _parse_color_as_rgb(color)
     red = min(max(int(red * ratio), 0), 255)
     green = min(max(int(green * ratio), 0), 255)
     blue = min(max(int(blue * ratio), 0), 255)
     return f'rgb({red}, {green}, {blue})'
 
 
-def lighten(color: Union[str, Color], percentage=10):
-    if isinstance(color, str) and color.startswith('rgb('):
-        color = literal_eval(color.lstrip('rgb(').rstrip(')'))
-    else:
-        color = color.as_rgb_tuple()
+def lighten(color: Union[str, Color], percentage: float = 10) -> str:
     ratio = float(percentage) / 100
-    red, green, blue = color
+    red, green, blue = _parse_color_as_rgb(color)
     red = min(max(int(red + (255 - red) * ratio), 0), 255)
     green = min(max(int(green + (255 - green) * ratio), 0), 255)
     blue = min(max(int(blue + (255 - blue) * ratio), 0), 255)
     return f'rgb({red}, {green}, {blue})'
 
 
-def opacity(color: Union[str, Color], value=255):
-    if isinstance(color, str) and color.startswith('rgb('):
-        color = literal_eval(color.lstrip('rgb(').rstrip(')'))
-    else:
-        color = color.as_rgb_tuple()
-    red, green, blue = color
+def opacity(color: Union[str, Color], value: int = 255) -> str:
+    red, green, blue = _parse_color_as_rgb(color)
     return f'rgba({red}, {green}, {blue}, {max(min(int(value), 255), 0)})'
 
 
-def gradient(stops, horizontal=True):
+def gradient(stops, horizontal: bool = True) -> str:
     if not use_gradients:
         return stops[-1]
 
@@ -145,12 +177,20 @@ def gradient(stops, horizontal=True):
         grad = 'qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, '
 
     _stops = [f'stop: {n} {stop}' for n, stop in enumerate(stops)]
-    grad += ", ".join(_stops) + ")"
+    grad += ', '.join(_stops) + ')'
 
     return grad
 
 
 def template(css: str, **theme):
+    def _increase_match(matchobj):
+        font_size, to_add = matchobj.groups()
+        return increase(theme[font_size], to_add)
+
+    def _decrease_match(matchobj):
+        font_size, to_subtract = matchobj.groups()
+        return decrease(theme[font_size], to_subtract)
+
     def darken_match(matchobj):
         color, percentage = matchobj.groups()
         return darken(theme[color], percentage)
@@ -169,13 +209,15 @@ def template(css: str, **theme):
         return gradient(stops, horizontal)
 
     for k, v in theme.items():
+        css = increase_pattern.sub(_increase_match, css)
+        css = decrease_pattern.sub(_decrease_match, css)
         css = gradient_pattern.sub(gradient_match, css)
         css = darken_pattern.sub(darken_match, css)
         css = lighten_pattern.sub(lighten_match, css)
         css = opacity_pattern.sub(opacity_match, css)
         if isinstance(v, Color):
             v = v.as_rgb()
-        css = css.replace('{{ %s }}' % k, v)
+        css = css.replace(f'{{{{ {k} }}}}', v)
     return css
 
 
@@ -184,12 +226,24 @@ def get_system_theme() -> str:
     try:
         id_ = darkdetect.theme().lower()
     except AttributeError:
-        id_ = "dark"
+        id_ = 'dark'
 
     return id_
 
 
-def get_theme(theme_id, as_dict=None):
+@overload
+def get_theme(theme_id: str) -> Theme: ...
+
+
+@overload
+def get_theme(theme_id: str, as_dict: Literal[False]) -> Theme: ...
+
+
+@overload
+def get_theme(theme_id: str, as_dict: Literal[True]) -> dict[str, Any]: ...
+
+
+def get_theme(theme_id: str, as_dict: Optional[bool] = None):
     """Get a copy of theme based on it's id.
 
     If you get a copy of the theme, changes to the theme model will not be
@@ -201,6 +255,10 @@ def get_theme(theme_id, as_dict=None):
     theme_id : str
         ID of requested theme.
     as_dict : bool
+        .. deprecated:: 0.5.0
+
+            Use ``get_theme(...).to_rgb_dict()``
+
         Flag to indicate that the old-style dictionary
         should be returned. This will emit deprecation warning.
 
@@ -211,39 +269,32 @@ def get_theme(theme_id, as_dict=None):
         so that manipulating this theme can be done without
         side effects.
     """
-    if theme_id == "system":
+    if theme_id == 'system':
         theme_id = get_system_theme()
 
     if theme_id not in _themes:
         raise ValueError(
             trans._(
-                "Unrecognized theme {id}. Available themes are {themes}",
+                'Unrecognized theme {id}. Available themes are {themes}',
                 deferred=True,
                 id=theme_id,
                 themes=available_themes(),
             )
         )
-    theme = _themes[theme_id]
-    _theme = theme.copy()
-    if as_dict is None:
+    theme = _themes[theme_id].copy()
+    if as_dict is not None:
         warnings.warn(
             trans._(
-                "The `as_dict` kwarg default to False` since Napari 0.4.17, "
-                "and will become a mandatory parameter in the future.",
+                'The `as_dict` kwarg has been deprecated since Napari 0.5.0 and '
+                'will be removed in future version. You can use `get_theme(...).to_rgb_dict()`',
                 deferred=True,
             ),
             category=FutureWarning,
             stacklevel=2,
         )
-        as_dict = False
     if as_dict:
-        _theme = _theme.dict()
-        _theme = {
-            k: v if not isinstance(v, Color) else v.as_rgb()
-            for (k, v) in _theme.items()
-        }
-        return _theme
-    return _theme
+        return theme.to_rgb_dict()
+    return theme
 
 
 _themes: EventedDict[str, Theme] = EventedDict(basetype=Theme)
@@ -280,7 +331,7 @@ def unregister_theme(theme_id):
     _themes.pop(theme_id, None)
 
 
-def available_themes() -> List[str]:
+def available_themes() -> list[str]:
     """List available themes.
 
     Returns
@@ -304,7 +355,7 @@ def is_theme_available(theme_id):
     bool
         True if the theme is available, False otherwise.
     """
-    if theme_id == "system":
+    if theme_id == 'system':
         return True
     if theme_id not in _themes and _theme_path(theme_id).exists():
         plugin_name_file = _theme_path(theme_id) / PLUGIN_FILE_NAME
@@ -346,6 +397,7 @@ DARK = Theme(
     syntax_style='native',
     console='rgb(18, 18, 18)',
     canvas='black',
+    font_size='12pt' if sys.platform == 'darwin' else '9pt',
 )
 LIGHT = Theme(
     id='light',
@@ -363,10 +415,11 @@ LIGHT = Theme(
     syntax_style='default',
     console='rgb(255, 255, 255)',
     canvas='white',
+    font_size='12pt' if sys.platform == 'darwin' else '9pt',
 )
 
-register_theme('dark', DARK, "builtin")
-register_theme('light', LIGHT, "builtin")
+register_theme('dark', DARK, 'builtin')
+register_theme('light', LIGHT, 'builtin')
 
 
 # this function here instead of plugins._npe2 to avoid circular import
@@ -389,7 +442,7 @@ def _install_npe2_themes(themes=None):
             try:
                 register_theme(theme.id, theme_dict, manifest.name)
             except ValueError:
-                logging.exception("Registration theme failed.")
+                logging.exception('Registration theme failed.')
 
 
 _install_npe2_themes(_themes)
