@@ -1,4 +1,4 @@
 from napari.utils.tree.group import Group
 from napari.utils.tree.node import Node
 
-__all__ = ["Node", "Group"]
+__all__ = ['Node', 'Group']
